@@ -1,27 +1,33 @@
+/// @file
+/// @ingroup common_render
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include <ctype.h>
 #include <common/render.h>
 #include <common/htmltable.h>
 #include <errno.h>
 #include <gvc/gvc.h>
 #include <xdot/xdot.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
-#include <cgraph/exit.h>
-#include <cgraph/startswith.h>
-#include <cgraph/strcasecmp.h>
+#include <limits.h>
 #include <stdbool.h>
-#include <stddef.h>
+#include <stdlib.h>
 #include <string.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/exit.h>
+#include <util/gv_ctype.h>
+#include <util/gv_fopen.h>
+#include <util/gv_math.h>
+#include <util/startswith.h>
+#include <util/strcasecmp.h>
+#include <util/streq.h>
 
 static char *usageFmt =
     "Usage: %s [-Vv?] [-(GNE)name=val] [-(KTlso)<val>] <dot files>\n";
@@ -58,23 +64,18 @@ static char *fdpItems = "\n\
  -LC<v>      - Set overlap expansion factor to v\n\
  -LT[*]<v>   - Set temperature (temperature factor) to v\n";
 
-static char *memtestFlags = "(additional options for memtest)  [-m<v>]\n";
-static char *memtestItems = "\n\
- -m          - Memory test (Observe no growth with top. Kill when done.)\n\
- -m[v]       - Memory test - v iterations.\n";
-
 static char *configFlags = "(additional options for config)  [-cv]\n";
 static char *configItems = "\n\
  -c          - Configure plugins (Writes $prefix/lib/graphviz/config \n\
                with available plugin information.  Needs write privilege.)\n\
  -?          - Print usage and exit\n";
 
-/* dotneato_usage:
- * Print usage information. If GvExitOnUsage is set, exit with
+/* Print usage information. If GvExitOnUsage is set, exit with
  * given exval, else return exval+1.
+ *
+ * @param argv0 Command name to use in usage message
  */
-int dotneato_usage(int exval)
-{
+int dotneato_usage(const char *argv0, int exval) {
     FILE *outs;
 
     if (exval > 0)
@@ -82,25 +83,21 @@ int dotneato_usage(int exval)
     else
 	outs = stdout;
 
-    fprintf(outs, usageFmt, CmdName);
+    fprintf(outs, usageFmt, argv0);
     fputs(neatoFlags, outs);
     fputs(fdpFlags, outs);
-    fputs(memtestFlags, outs);
     fputs(configFlags, outs);
     fputs(genericItems, outs);
     fputs(neatoItems, outs);
     fputs(fdpItems, outs);
-    fputs(memtestItems, outs);
     fputs(configItems, outs);
 
-    if (GvExitOnUsage && (exval >= 0))
+    if (GvExitOnUsage && exval >= 0)
 	graphviz_exit(exval);
-    return (exval+1);
-	
+    return exval + 1;
 }
 
-/* getFlagOpt:
- * Look for flag parameter. idx is index of current argument.
+/* Look for flag parameter. idx is index of current argument.
  * We assume argv[*idx] has the form "-x..." If there are characters 
  * after the x, return
  * these, else if there are more arguments, return the next one,
@@ -116,7 +113,7 @@ static char *getFlagOpt(int argc, char **argv, int *idx)
     if (i < argc - 1) {
 	i++;
 	arg = argv[i];
-	if (*arg && (*arg != '-')) {
+	if (*arg && *arg != '-') {
 	    *idx = i;
 	    return arg;
 	}
@@ -124,17 +121,15 @@ static char *getFlagOpt(int argc, char **argv, int *idx)
     return 0;
 }
 
-/* dotneato_basename:
- * Partial implementation of real basename.
+/* Partial implementation of real basename.
  * Skip over any trailing slashes or backslashes; then
  * find next (back)slash moving left; return string to the right.
  * If no next slash is found, return the whole string.
  */
-static char* dotneato_basename (char* path)
-{
+static char *dotneato_basename(char *pathname) {
     char* ret;
-    char* s = path;
-    if (*s == '\0') return path; /* empty string */
+    char* s = pathname;
+    if (*s == '\0') return pathname; /* empty string */
 #ifdef _WIN32
     /* On Windows, executables, by convention, end in ".exe". Thus,
      * this may be part of the path name and must be removed for
@@ -148,22 +143,18 @@ static char* dotneato_basename (char* path)
     while (*s) s++;
     s--;
     /* skip over trailing slashes, nulling out as we go */
-    while (s > path && (*s == '/' || *s == '\\'))
+    while (s > pathname && (*s == '/' || *s == '\\'))
 	*s-- = '\0';
-    if (s == path) ret = path;
+    if (s == pathname) ret = pathname;
     else {
-	while (s > path && (*s != '/' && *s != '\\')) s--;
+	while (s > pathname && *s != '/' && *s != '\\') s--;
 	if (*s == '/' || *s == '\\') ret = s+1;
-	else ret = path;
+	else ret = pathname;
     }
 #ifdef _WIN32
     /* On Windows, names are case-insensitive, so make name lower-case
      */
-    {
-	char c;
-	for (s = ret; (c = *s); s++)
-	    *s = tolower(c);
-    }
+    gv_tolower_str(ret);
 #endif
     return ret;
 }
@@ -172,7 +163,8 @@ static void use_library(GVC_t *gvc, const char *name)
 {
     static size_t cnt = 0;
     if (name) {
-	Lib = ALLOC(cnt + 2, Lib, const char *);
+	const size_t old_nmemb = cnt == 0 ? cnt : cnt + 1;
+	Lib = gv_recalloc(Lib, old_nmemb, cnt + 2, sizeof(const char *));
 	Lib[cnt++] = name;
 	Lib[cnt] = NULL;
     }
@@ -191,16 +183,14 @@ static void global_def(char *dcl, int kind) {
     }
     else
 	agxbput(&xb, dcl);
-    sym = agattr(NULL, kind, agxbuse(&xb), rhs);
+    sym = agattr_text(NULL, kind, agxbuse(&xb), rhs);
     sym->fixed = 1;
     agxbfree(&xb);
 }
 
 static int gvg_init(GVC_t *gvc, graph_t *g, char *fn, int gidx)
 {
-    GVG_t *gvg;
-
-    gvg = zmalloc(sizeof(GVG_t));
+    GVG_t *gvg = gv_alloc(sizeof(GVG_t));
     if (!gvc->gvgs) 
 	gvc->gvgs = gvg;
     else
@@ -221,17 +211,14 @@ graph_t *gvPluginsGraph(GVC_t *gvc)
     return P_graph;
 }
 
-/* dotneato_args_initialize"
- * Scan argv[] for allowed flags.
+/* Scan argv[] for allowed flags.
  * Return 0 on success; v+1 if calling function should call exit(v).
  * If -c is set, config file is created and we exit. 
  */
 int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 {
-    char c, *rest, *layout;
-    const char *val;
-    int i, v, nfiles;
-    int Kflag = 0;
+    char c;
+    bool Kflag = false;
 
     /* establish if we are running in a CGI environment */
     HTTPServerEnVar = getenv("SERVER_NAME");
@@ -260,16 +247,17 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
     }
 
     /* feed the globals */
-    Verbose = gvc->common.verbose;
-    CmdName = gvc->common.cmdname;
+    Verbose = gvc->common.verbose > UCHAR_MAX
+      ? UCHAR_MAX
+      : (unsigned char)gvc->common.verbose;
 
-    nfiles = 0;
-    for (i = 1; i < argc; i++)
+    size_t nfiles = 0;
+    for (int i = 1; i < argc; i++)
 	if (argv[i] && argv[i][0] != '-')
 	    nfiles++;
-    gvc->input_filenames = N_NEW(nfiles + 1, char *);
+    gvc->input_filenames = gv_calloc(nfiles + 1, sizeof(char *));
     nfiles = 0;
-    for (i = 1; i < argc; i++) {
+    for (int i = 1; i < argc; i++) {
 	if (argv[i] &&
 	    (startswith(argv[i], "-V") || strcmp(argv[i], "--version") == 0)) {
 	    fprintf(stderr, "%s - %s version %s (%s)\n",
@@ -279,16 +267,19 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 	    return 1;
 	} else if (argv[i] &&
 	    (startswith(argv[i], "-?") || strcmp(argv[i], "--help") == 0)) {
-	    return dotneato_usage(0);
+	    return dotneato_usage(argv[0], 0);
+	} else if (argv[i] && startswith(argv[i], "--filepath=")) {
+	    free(Gvfilepath);
+	    Gvfilepath = gv_strdup(argv[i] + strlen("--filepath="));
 	} else if (argv[i] && argv[i][0] == '-') {
-	    rest = &argv[i][2];
+	    char *const rest = &argv[i][2];
 	    switch (c = argv[i][1]) {
 	    case 'G':
 		if (*rest)
 		    global_def(rest, AGRAPH);
 		else {
 		    fprintf(stderr, "Missing argument for -G flag\n");
-		    return (dotneato_usage(1));
+		    return dotneato_usage(argv[0], 1);
 		}
 		break;
 	    case 'N':
@@ -296,7 +287,7 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 		    global_def(rest, AGNODE);
 		else {
 		    fprintf(stderr, "Missing argument for -N flag\n");
-		    return (dotneato_usage(1));
+		    return dotneato_usage(argv[0], 1);
 		}
 		break;
 	    case 'E':
@@ -304,20 +295,18 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 		    global_def(rest, AGEDGE);
 		else {
 		    fprintf(stderr, "Missing argument for -E flag\n");
-		    return (dotneato_usage(1));
+		    return dotneato_usage(argv[0], 1);
 		}
 		break;
-	    case 'T':
-		val = getFlagOpt(argc, argv, &i);
+	    case 'T': {
+		const char *const val = getFlagOpt(argc, argv, &i);
 		if (!val) {
 		    fprintf(stderr, "Missing argument for -T flag\n");
-		    return (dotneato_usage(1));
+		    return dotneato_usage(argv[0], 1);
 		}
 		if (!gvjobs_output_langname(gvc, val)) {
-		    /* TODO: Detect empty results from gvplugin_list() and prompt to configure with '-c' */
-		    char *fmts;
+		    const char *const fmts = gvplugin_list(gvc, API_device, val);
 		    fprintf(stderr, "Format: \"%s\" not recognized.", val);
-		    fmts = gvplugin_list(gvc, API_device, val);
 		    if (strlen(fmts) > 1) {
 			fprintf(stderr, " Use one of:%s\n", fmts);
 		    } else {
@@ -325,26 +314,24 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 			fprintf(stderr, " No formats found.\nPerhaps \"dot -c\" needs to be run (with installer's privileges) to register the plugins?\n");
 		    }
 		    if (GvExitOnUsage) graphviz_exit(1);
-		    return(2);
+		    return 2;
 		}
 		break;
-	    case 'K':
-		val = getFlagOpt(argc, argv, &i);
+	    }
+	    case 'K': {
+		const char *const val = getFlagOpt(argc, argv, &i);
 		if (!val) {
                     fprintf(stderr, "Missing argument for -K flag\n");
-                    return (dotneato_usage(1));
+                    return dotneato_usage(argv[0], 1);
                 }
-                v = gvlayout_select(gvc, val);
+                const int v = gvlayout_select(gvc, val);
                 if (v == NO_SUPPORT) {
 	            fprintf(stderr, "There is no layout engine support for \"%s\"\n", val);
                     if (streq(val, "dot")) {
                         fprintf(stderr, "Perhaps \"dot -c\" needs to be run (with installer's privileges) to register the plugins?\n");
                     }
 		    else {
-			/* TODO: Detect empty results from gvplugin_list() and prompt to configure with '-c' */
-			/* fprintf(stderr, "Use one of:%s\n", gvplugin_list(gvc, API_layout, val)); */
-			char *lyts;
-			lyts = gvplugin_list(gvc, API_layout, val);
+			const char *const lyts = gvplugin_list(gvc, API_layout, val);
 			if (strlen(lyts) > 1) {
 			    fprintf(stderr, " Use one of:%s\n", lyts);
 			} else {
@@ -353,33 +340,36 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 			}
 		    }
 		    if (GvExitOnUsage) graphviz_exit(1);
-		    return(2);
+		    return 2;
                 }
-		Kflag = 1;
+		Kflag = true;
 		break;
+	    }
 	    case 'P':
 		P_graph = gvplugin_graph(gvc);
 		break;
-	    case 'l':
-		val = getFlagOpt(argc, argv, &i);
+	    case 'l': {
+		const char *const val = getFlagOpt(argc, argv, &i);
 		if (!val) {
 		    fprintf(stderr, "Missing argument for -l flag\n");
-		    return (dotneato_usage(1));
+		    return dotneato_usage(argv[0], 1);
 		}
 		use_library(gvc, val);
 		break;
-	    case 'o':
-		val = getFlagOpt(argc, argv, &i);
+	    }
+	    case 'o': {
+		const char *const val = getFlagOpt(argc, argv, &i);
 		if (!val) {
 		    fprintf(stderr, "Missing argument for -o flag\n");
-		    return (dotneato_usage(1));
+		    return dotneato_usage(argv[0], 1);
 		}
 		if (! gvc->common.auto_outfile_names)
 		    gvjobs_output_filename(gvc, val);
 		break;
+	    }
 	    case 'q':
 		if (*rest) {
-		    v = atoi(rest);
+		    const int v = atoi(rest);
 		    if (v <= 0) {
 			fprintf(stderr,
 				"Invalid parameter \"%s\" for -q flag - ignored\n",
@@ -398,23 +388,23 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 			fprintf(stderr,
 				"Invalid parameter \"%s\" for -s flag\n",
 				rest);
-			return (dotneato_usage(1));
+			return dotneato_usage(argv[0], 1);
 		    }
-		    else if (PSinputscale == 0)
+		    else if (is_exactly_zero(PSinputscale))
 			PSinputscale = POINTS_PER_INCH;
 		} else
 		    PSinputscale = POINTS_PER_INCH;
 		break;
 	    case 'x':
-		Reduce = TRUE;
+		Reduce = true;
 		break;
 	    case 'y':
-		Y_invert = TRUE;
+		Y_invert = true;
 		break;
 	    default:
-		agerr(AGERR, "%s: option -%c unrecognized\n\n", gvc->common.cmdname,
+		agerrorf("%s: option -%c unrecognized\n\n", gvc->common.cmdname,
 			c);
-		return (dotneato_usage(1));
+		return dotneato_usage(argv[0], 1);
 	    }
 	} else if (argv[i])
 	    gvc->input_filenames[nfiles++] = argv[i];
@@ -422,7 +412,7 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 
     /* if no -K, use cmd name to set layout type */
     if (!Kflag) {
-	layout = gvc->common.cmdname;
+	const char *layout = gvc->common.cmdname;
 	if (streq(layout, "dot_static")
 	    || streq(layout, "dot_builtins")
 	    || streq(layout, "lt-dot")
@@ -430,16 +420,13 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 	    || streq(layout, "")   /* when run as a process from Gvedit on Windows */
 	)
             layout = "dot";
-	i = gvlayout_select(gvc, layout);
+	const int i = gvlayout_select(gvc, layout);
 	if (i == NO_SUPPORT) {
 	    fprintf(stderr, "There is no layout engine support for \"%s\"\n", layout);
             if (streq(layout, "dot")) {
 		fprintf(stderr, "Perhaps \"dot -c\" needs to be run (with installer's privileges) to register the plugins?\n");
 	    } else {
-		/* TODO: Detect empty results from gvplugin_list() and prompt to configure with '-c' */
-		/* fprintf(stderr, "Use one of:%s\n", gvplugin_list(gvc, API_layout, "")); */
-		char *lyts;
-		lyts = gvplugin_list(gvc, API_layout, "");
+		const char *const lyts = gvplugin_list(gvc, API_layout, "");
 		if (strlen(lyts) > 1) {
                     fprintf(stderr, " Use one of:%s\n", lyts);
 		} else {
@@ -449,7 +436,7 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 	    }
 
 	    if (GvExitOnUsage) graphviz_exit(1);
-	    return(2);
+	    return 2;
 	}
     }
 
@@ -458,18 +445,17 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 	if (!gvjobs_output_langname(gvc, "dot")) {
 		fprintf(stderr,
 			"Unable to find even the default \"-Tdot\" renderer.  Has the config\nfile been generated by running \"dot -c\" with installer's privileges?\n");
-			return(2);
+			return 2;
 	}
     }
 
     /* set persistent attributes here (if not already set from command line options) */
-    if (!agattr(NULL, AGNODE, "label", 0))
-	agattr(NULL, AGNODE, "label", NODENAME_ESC);
+    if (!agattr_text(NULL, AGNODE, "label", 0))
+	agattr_text(NULL, AGNODE, "label", NODENAME_ESC);
     return 0;
 }
 
-/* getdoubles2ptf:
- * converts a graph attribute in inches to a pointf in points.
+/* converts a graph attribute in inches to a pointf in points.
  * If only one number is given, it is used for both x and y.
  * Returns true if the attribute ends in '!'.
  */
@@ -516,49 +502,41 @@ graph_t *gvNextInputGraph(GVC_t *gvc)
     graph_t *g = NULL;
     static char *fn;
     static FILE *fp;
-    static FILE *oldfp;
-    static int fidx, gidx;
+    static int gidx;
 
     while (!g) {
 	if (!fp) {
     	    if (!(fn = gvc->input_filenames[0])) {
-		if (fidx++ == 0)
+		if (gvc->fidx++ == 0)
 		    fp = stdin;
 	    }
 	    else {
-		while ((fn = gvc->input_filenames[fidx++]) && !(fp = fopen(fn, "r")))  {
-		    agerr(AGERR, "%s: can't open %s: %s\n", gvc->common.cmdname, fn, strerror(errno));
+		while ((fn = gvc->input_filenames[gvc->fidx++]) && !(fp = gv_fopen(fn, "r")))  {
+		    agerrorf("%s: can't open %s: %s\n", gvc->common.cmdname, fn, strerror(errno));
 		    graphviz_errors++;
 		}
 	    }
 	}
 	if (fp == NULL)
 	    break;
-	if (oldfp != fp) {
-	    agsetfile(fn ? fn : "<stdin>");
-	    oldfp = fp;
-	}
-	g = agread(fp,NULL);
+	g = agconcat(NULL, fn ? fn : "<stdin>", fp, NULL);
 	if (g) {
 	    gvg_init(gvc, g, fn, gidx++);
 	    break;
 	}
 	if (fp != stdin)
 	    fclose (fp);
-	oldfp = fp = NULL;
+	fp = NULL;
 	gidx = 0;
     }
     return g;
 }
 
-/* findCharset:
- * Check if the charset attribute is defined for the graph and, if
+/* Check if the charset attribute is defined for the graph and, if
  * so, return the corresponding internal value. If undefined, return
  * CHAR_UTF8
  */
-static int findCharset (graph_t * g)
-{
-    int enc;
+static unsigned char findCharset(graph_t *g) {
     char* p;
 
     p = late_nnstring(g,agfindgraphattr(g,"charset"),"utf-8");
@@ -569,61 +547,42 @@ static int findCharset (graph_t * g)
 	|| !strcasecmp(p,"ISO_8859-1")
 	|| !strcasecmp(p,"ISO8859-1")
 	|| !strcasecmp(p,"ISO-IR-100"))
-		enc = CHAR_LATIN1; 
-    else if (!strcasecmp(p,"big-5")
+      return CHAR_LATIN1;
+    if (!strcasecmp(p,"big-5")
 	|| !strcasecmp(p,"big5")) 
-		enc = CHAR_BIG5; 
-    else if (!strcasecmp(p,"utf-8")
+      return CHAR_BIG5;
+    if (!strcasecmp(p,"utf-8")
 	|| !strcasecmp(p,"utf8"))
-		enc = CHAR_UTF8; 
-    else {
-	agerr(AGWARN, "Unsupported charset \"%s\" - assuming utf-8\n", p);
-	enc = CHAR_UTF8; 
-    }
-    return enc;
+      return CHAR_UTF8;
+    agwarningf("Unsupported charset \"%s\" - assuming utf-8\n", p);
+    return CHAR_UTF8;
 }
 
-/* setRatio:
- * Checks "ratio" attribute, if any, and sets enum type.
- */
+/// Checks "ratio" attribute, if any, and sets enum type.
 static void setRatio(graph_t * g)
 {
-    char *p, c;
+    char *p;
     double ratio;
 
-    if ((p = agget(g, "ratio")) && (c = p[0])) {
-	switch (c) {
-	case 'a':
-	    if (streq(p, "auto"))
-		GD_drawing(g)->ratio_kind = R_AUTO;
-	    break;
-	case 'c':
-	    if (streq(p, "compress"))
-		GD_drawing(g)->ratio_kind = R_COMPRESS;
-	    break;
-	case 'e':
-	    if (streq(p, "expand"))
-		GD_drawing(g)->ratio_kind = R_EXPAND;
-	    break;
-	case 'f':
-	    if (streq(p, "fill"))
-		GD_drawing(g)->ratio_kind = R_FILL;
-	    break;
-	default:
+    if ((p = agget(g, "ratio"))) {
+	if (streq(p, "auto")) {
+	    GD_drawing(g)->ratio_kind = R_AUTO;
+	} else if (streq(p, "compress")) {
+	    GD_drawing(g)->ratio_kind = R_COMPRESS;
+	} else if (streq(p, "expand")) {
+	    GD_drawing(g)->ratio_kind = R_EXPAND;
+	} else if (streq(p, "fill")) {
+	    GD_drawing(g)->ratio_kind = R_FILL;
+	} else {
 	    ratio = atof(p);
 	    if (ratio > 0.0) {
 		GD_drawing(g)->ratio_kind = R_VALUE;
 		GD_drawing(g)->ratio = ratio;
 	    }
-	    break;
 	}
     }
 }
 
-/*
-	cgraph requires 
-
-*/
 void graph_init(graph_t * g, bool use_rankdir)
 {
     char *p;
@@ -660,6 +619,9 @@ void graph_init(graph_t * g, bool use_rankdir)
 
     if (!HTTPServerEnVar) {
 	Gvimagepath = agget (g, "imagepath");
+	if (!Gvimagepath) {
+	    Gvimagepath = Gvfilepath;
+	}
     }
 
     GD_drawing(g)->quantum =
@@ -684,7 +646,7 @@ void graph_init(graph_t * g, bool use_rankdir)
     if (use_rankdir)
 	SET_RANKDIR (g, (rankdir << 2) | rankdir);
     else
-	SET_RANKDIR (g, (rankdir << 2));
+	SET_RANKDIR(g, rankdir << 2);
 
     xf = late_double(g, agfindgraphattr(g, "nodesep"),
 		DEFAULT_NODESEP, MIN_NODESEP);
@@ -704,13 +666,19 @@ void graph_init(graph_t * g, bool use_rankdir)
 	xf = DEFAULT_RANKSEP;
     GD_ranksep(g) = POINTS(xf);
 
-    GD_showboxes(g) = late_int(g, agfindgraphattr(g, "showboxes"), 0, 0);
+    {
+	int showboxes = late_int(g, agfindgraphattr(g, "showboxes"), 0, 0);
+	if (showboxes > UCHAR_MAX) {
+	    showboxes = UCHAR_MAX;
+	}
+	GD_showboxes(g) = (unsigned char)showboxes;
+    }
     p = late_string(g, agfindgraphattr(g, "fontnames"), NULL);
     GD_fontnames(g) = maptoken(p, fontnamenames, fontnamecodes);
 
     setRatio(g);
     GD_drawing(g)->filled = getdoubles2ptf(g, "size", &GD_drawing(g)->size);
-    getdoubles2ptf(g, "page", &(GD_drawing(g)->page));
+    getdoubles2ptf(g, "page", &GD_drawing(g)->page);
 
     GD_drawing(g)->centered = mapbool(agget(g, "center"));
 
@@ -724,7 +692,7 @@ void graph_init(graph_t * g, bool use_rankdir)
     p = agget(g, "clusterrank");
     CL_type = maptoken(p, rankname, rankcode);
     p = agget(g, "concentrate");
-    Concentrate = mapbool(p) ? TRUE : FALSE;
+    Concentrate = mapbool(p);
     State = GVBEGIN;
     EdgeLabelsDone = 0;
 
@@ -753,12 +721,11 @@ void graph_init(graph_t * g, bool use_rankdir)
     N_fontcolor = agfindnodeattr(g, "fontcolor");
     N_label = agfindnodeattr(g, "label");
     if (!N_label)
-	N_label = agattr(g, AGNODE, "label", NODENAME_ESC);
+	N_label = agattr_text(g, AGNODE, "label", NODENAME_ESC);
     N_xlabel = agfindnodeattr(g, "xlabel");
     N_showboxes = agfindnodeattr(g, "showboxes");
     N_penwidth = agfindnodeattr(g, "penwidth");
     N_ordering = agfindnodeattr(g, "ordering");
-    N_margin = agfindnodeattr(g, "margin");
     /* attribs for polygon shapes */
     N_sides = agfindnodeattr(g, "sides");
     N_peripheries = agfindnodeattr(g, "peripheries");
@@ -787,8 +754,6 @@ void graph_init(graph_t * g, bool use_rankdir)
     E_xlabel = agfindedgeattr(g, "xlabel");
     E_label_float = agfindedgeattr(g, "labelfloat");
     E_dir = agfindedgeattr(g, "dir");
-    E_arrowhead = agfindedgeattr(g, "arrowhead");
-    E_arrowtail = agfindedgeattr(g, "arrowtail");
     E_headlabel = agfindedgeattr(g, "headlabel");
     E_taillabel = agfindedgeattr(g, "taillabel");
     E_labelfontsize = agfindedgeattr(g, "labelfontsize");
@@ -830,10 +795,7 @@ void graph_cleanup(graph_t *g)
     agclean(g, AGRAPH,"Agraphinfo_t");
 }
 
-/* charsetToStr:
- * Given an internal charset value, return a canonical string
- * representation.
- */
+/// Given an internal charset value, return a canonical string representation.
 char*
 charsetToStr (int c)
 {
@@ -850,17 +812,14 @@ charsetToStr (int c)
 	s = "BIG-5";
 	break;
    default :
-	agerr(AGERR, "Unsupported charset value %d\n", c);
+	agerrorf("Unsupported charset value %d\n", c);
 	s = "UTF-8";
 	break;
    }
    return s;
 }
 
-/* do_graph_label:
- * Set characteristics of graph label if it exists.
- * 
- */
+/// Set characteristics of graph label if it exists.
 void do_graph_label(graph_t * sg)
 {
     char *str, *pos, *just;
