@@ -1,45 +1,45 @@
+/// @file
+/// @ingroup common_render
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include "config.h"
 #include <assert.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
 #include <common/geomprocs.h>
 #include <common/render.h>
+#include <float.h>
 #include <limits.h>
 #include <math.h>
 #include <pathplan/pathplan.h>
-#include <setjmp.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/gv_math.h>
+#include <util/list.h>
+#include <util/prisize_t.h>
 
-static int nedges, nboxes; /* total no. of edges and boxes used in routing */
+static int nedges; ///< total no. of edges used in routing
+static size_t nboxes; ///< total no. of boxes used in routing
 
 static int routeinit;
-/* static data used across multiple edges */
-static Ppoint_t *polypoints;  /* vertices of polygon defined by boxes */
-static int polypointn;        /* size of polypoints[] */
-static Pedge_t *edges;        /* polygon edges passed to Proutespline */
-static int edgen;             /* size of edges[] */
 
-static int checkpath(int, boxf*, path*);
+static int checkpath(size_t, boxf *, path *);
 static void printpath(path * pp);
 #ifdef DEBUG
-static void printboxes(int boxn, boxf* boxes)
-{
+static void printboxes(size_t boxn, boxf *boxes) {
     pointf ll, ur;
-    int bi;
 
-    for (bi = 0; bi < boxn; bi++) {
+    for (size_t bi = 0; bi < boxn; bi++) {
 	ll = boxes[bi].LL, ur = boxes[bi].UR;
 	agxbuf buf = {0};
 	agxbprint(&buf, "%.0f %.0f %.0f %.0f pathbox", ll.x, ll.y, ur.x, ur.y);
@@ -85,12 +85,10 @@ static void psprintpointf(pointf p)
 
 static void psprintspline(Ppolyline_t spl)
 {
-    int i;
-
     show_boxes_append(&Show_boxes, gv_strdup("%%!"));
     show_boxes_append(&Show_boxes, gv_strdup("%% spline"));
     show_boxes_append(&Show_boxes, gv_strdup("gsave 1 0 0 setrgbcolor newpath"));
-    for (i = 0; i < spl.pn; i++) {
+    for (size_t i = 0; i < spl.pn; i++) {
 	agxbuf buf = {0};
 	agxbprint(&buf, "%f %f %s", spl.ps[i].x, spl.ps[i].y,
 	  i == 0 ?  "moveto" : (i % 3 == 0 ? "curveto" : ""));
@@ -101,12 +99,10 @@ static void psprintspline(Ppolyline_t spl)
 
 static void psprintline(Ppolyline_t pl)
 {
-    int i;
-
     show_boxes_append(&Show_boxes, gv_strdup("%%!"));
     show_boxes_append(&Show_boxes, gv_strdup("%% line"));
     show_boxes_append(&Show_boxes, gv_strdup("gsave 0 0 1 setrgbcolor newpath"));
-    for (i = 0; i < pl.pn; i++) {
+    for (size_t i = 0; i < pl.pn; i++) {
 	agxbuf buf = {0};
 	agxbprint(&buf, "%f %f %s", pl.ps[i].x, pl.ps[i].y,
 		i == 0 ? "moveto" : "lineto");
@@ -117,34 +113,29 @@ static void psprintline(Ppolyline_t pl)
 
 static void psprintpoly(Ppoly_t p)
 {
-    point tl, hd;
-    int bi;
     char*  pfx;
 
     show_boxes_append(&Show_boxes, gv_strdup("%% poly list"));
     show_boxes_append(&Show_boxes, gv_strdup("gsave 0 1 0 setrgbcolor"));
-    for (bi = 0; bi < p.pn; bi++) {
-	tl.x = (int)p.ps[bi].x;
-	tl.y = (int)p.ps[bi].y;
-	hd.x = (int)p.ps[(bi+1) % p.pn].x;
-	hd.y = (int)p.ps[(bi+1) % p.pn].y;
-	if (tl.x == hd.x && tl.y == hd.y) pfx = "%%";
+    for (size_t bi = 0; bi < p.pn; bi++) {
+	const pointf tail = p.ps[bi];
+	const pointf head = p.ps[(bi + 1) % p.pn];
+	if (fabs(tail.x - head.x) < 1 && fabs(tail.y - head.y) < 1) pfx = "%%";
 	else pfx ="";
 	agxbuf buf = {0};
-	agxbprint(&buf, "%s%d %d %d %d makevec", pfx, tl.x, tl.y, hd.x, hd.y);
+	agxbprint(&buf, "%s%.0f %.0f %.0f %.0f makevec", pfx, tail.x, tail.y, head.x,
+	          head.y);
 	show_boxes_append(&Show_boxes, agxbdisown(&buf));
     }
     show_boxes_append(&Show_boxes, gv_strdup("grestore"));
 }
 
-static void psprintboxes(int boxn, boxf* boxes)
-{
+static void psprintboxes(size_t boxn, boxf *boxes) {
     pointf ll, ur;
-    int bi;
 
     show_boxes_append(&Show_boxes, gv_strdup("%% box list"));
     show_boxes_append(&Show_boxes, gv_strdup("gsave 0 1 0 setrgbcolor"));
-    for (bi = 0; bi < boxn; bi++) {
+    for (size_t bi = 0; bi < boxn; bi++) {
 	ll = boxes[bi].LL, ur = boxes[bi].UR;
 	agxbuf buf = {0};
 	agxbprint(&buf, "newpath\n%.0f %.0f moveto", ll.x, ll.y);
@@ -178,19 +169,12 @@ static bool debugleveln(edge_t* realedge, int i)
 }
 #endif  /* DEBUG */
 
-
-
-/* simpleSplineRoute:
- * Given a simple (ccw) polygon, route an edge from tp to hp.
- */
-pointf*
-simpleSplineRoute (pointf tp, pointf hp, Ppoly_t poly, int* n_spl_pts,
-    int polyline)
-{
+/// Given a simple (ccw) polygon, route an edge from tp to hp.
+pointf *simpleSplineRoute(pointf tp, pointf hp, Ppoly_t poly, size_t *n_spl_pts,
+                          int polyline) {
     Ppolyline_t pl, spl;
     Ppoint_t eps[2];
     Pvector_t evs[2];
-    int i;
 
     eps[0].x = tp.x;
     eps[0].y = tp.y;
@@ -202,38 +186,38 @@ simpleSplineRoute (pointf tp, pointf hp, Ppoly_t poly, int* n_spl_pts,
     if (polyline)
 	make_polyline (pl, &spl);
     else {
-	if (poly.pn > edgen) {
-	    edges = ALLOC(poly.pn, edges, Pedge_t);
-	    edgen = poly.pn;
-	}
-	for (i = 0; i < poly.pn; i++) {
+	// polygon edges passed to Proutespline
+	Pedge_t *edges = gv_calloc(poly.pn, sizeof(Pedge_t));
+	for (size_t i = 0; i < poly.pn; i++) {
 	    edges[i].a = poly.ps[i];
 	    edges[i].b = poly.ps[(i + 1) % poly.pn];
 	}
 	    evs[0].x = evs[0].y = 0;
 	    evs[1].x = evs[1].y = 0;
-	if (Proutespline(edges, poly.pn, pl, evs, &spl) < 0)
+	if (Proutespline(edges, poly.pn, pl, evs, &spl) < 0) {
+            free(edges);
             return NULL;
+	}
+	free(edges);
     }
 
     pointf *ps = calloc(spl.pn, sizeof(ps[0]));
     if (ps == NULL) {
-	agerr(AGERR, "cannot allocate ps\n");
+	agerrorf("cannot allocate ps\n");
 	return NULL;
     }
-    for (i = 0; i < spl.pn; i++) {
+    for (size_t i = 0; i < spl.pn; i++) {
         ps[i] = spl.ps[i];
     }
     *n_spl_pts = spl.pn;
     return ps;
 }
 
-/* routesplinesinit:
- * Data initialized once until matching call to routeplineterm
+/** Data initialized once until matching call to routeplineterm
  * Allows recursive calls to dot
  */
 int
-routesplinesinit()
+routesplinesinit(void)
 {
     if (++routeinit > 1) return 0;
 #ifdef DEBUG
@@ -246,26 +230,24 @@ routesplinesinit()
     return 0;
 }
 
-void routesplinesterm()
+void routesplinesterm(void)
 {
     if (--routeinit > 0) return;
     if (Verbose)
 	fprintf(stderr,
-		"routesplines: %d edges, %d boxes %.2f sec\n",
+		"routesplines: %d edges, %" PRISIZE_T " boxes %.2f sec\n",
 		nedges, nboxes, elapsed_sec());
 }
 
-static void
-limitBoxes (boxf* boxes, int boxn, const pointf *pps, int pn, int delta)
-{
-    int bi, si, splinepi;
+static void limitBoxes(boxf *boxes, size_t boxn, const pointf *pps, size_t pn,
+                       double delta) {
     double t;
     pointf sp[4];
-    int num_div = delta * boxn;
+    const double num_div = delta * (double)boxn;
 
-    for (splinepi = 0; splinepi + 3 < pn; splinepi += 3) {
-	for (si = 0; si <= num_div; si++) {
-	    t = si / (double)num_div;
+    for (size_t splinepi = 0; splinepi + 3 < pn; splinepi += 3) {
+	for (double si = 0; si <= num_div; si++) {
+	    t = si / num_div;
 	    sp[0] = pps[splinepi];
 	    sp[1] = pps[splinepi + 1];
 	    sp[2] = pps[splinepi + 2];
@@ -282,7 +264,7 @@ limitBoxes (boxf* boxes, int boxn, const pointf *pps, int pn, int delta)
 	    sp[1].y += t * (sp[2].y - sp[1].y);
 	    sp[0].x += t * (sp[1].x - sp[0].x);
 	    sp[0].y += t * (sp[1].y - sp[0].y);
-	    for (bi = 0; bi < boxn; bi++) {
+	    for (size_t bi = 0; bi < boxn; bi++) {
 /* this tested ok on 64bit machines, but on 32bit we need this FUDGE
  *     or graphs/directed/records.gv fails */
 #define FUDGE .0001
@@ -298,8 +280,7 @@ limitBoxes (boxf* boxes, int boxn, const pointf *pps, int pn, int delta)
 #define INIT_DELTA 10 
 #define LOOP_TRIES 15  /* number of times to try to limiting boxes to regain space, using smaller divisions */
 
-/* routesplines:
- * Route a path using the path info in pp. This includes start and end points
+/** Route a path using the path info in pp. This includes start and end points
  * plus a collection of contiguous boxes contain the terminal points. The boxes
  * are converted into a containing polygon. A shortest path is constructed within
  * the polygon from between the terminal points. If polyline is true, this path
@@ -317,20 +298,16 @@ limitBoxes (boxf* boxes, int boxn, const pointf *pps, int pn, int delta)
  *
  * If a catastrophic error, return NULL and npoints is 0.
  */
-static pointf *_routesplines(path * pp, int *npoints, int polyline)
-{
+static pointf *routesplines_(path *pp, size_t *npoints, int polyline) {
     Ppoly_t poly;
     Ppolyline_t pl, spl;
-    int splinepi;
     Ppoint_t eps[2];
     Pvector_t evs[2];
-    int edgei, prev, next;
-    int pi, bi;
+    int prev, next;
     boxf *boxes;
-    int boxn;
     edge_t* realedge;
     bool flip;
-    int loopcnt, delta = INIT_DELTA;
+    int loopcnt;
     bool unbounded;
 
     *npoints = 0;
@@ -341,12 +318,12 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	 realedge && ED_edge_type(realedge) != NORMAL;
 	 realedge = ED_to_orig(realedge));
     if (!realedge) {
-	agerr(AGERR, "in routesplines, cannot find NORMAL edge\n");
+	agerrorf("in routesplines, cannot find NORMAL edge\n");
 	return NULL;
     }
 
     boxes = pp->boxes;
-    boxn = pp->nbox;
+    const size_t boxn = pp->nbox;
 
     if (checkpath(boxn, boxes, pp))
 	return NULL;
@@ -360,14 +337,12 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
     }
 #endif
 
-    if (boxn * 8 > polypointn) {
-	polypoints = ALLOC(boxn * 8, polypoints, Ppoint_t);
-	polypointn = boxn * 8;
-    }
+    // vertices of polygon defined by boxes
+    Ppoint_t *polypoints = gv_calloc(boxn * 8, sizeof(Ppoint_t));
 
     if (boxn > 1 && boxes[0].LL.y > boxes[1].LL.y) {
         flip = true;
-	for (bi = 0; bi < boxn; bi++) {
+	for (size_t bi = 0; bi < boxn; bi++) {
 	    double v = boxes[bi].UR.y;
 	    boxes[bi].UR.y = -1*boxes[bi].LL.y;
 	    boxes[bi].LL.y = -v;
@@ -375,14 +350,16 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
     }
     else flip = false;
 
+    size_t pi;
     if (agtail(realedge) != aghead(realedge)) {
 	/* I assume that the path goes either down only or
 	   up - right - down */
+	size_t bi;
 	for (bi = 0, pi = 0; bi < boxn; bi++) {
 	    next = prev = 0;
 	    if (bi > 0)
 		prev = boxes[bi].LL.y > boxes[bi - 1].LL.y ? -1 : 1;
-	    if (bi < boxn - 1)
+	    if (bi + 1 < boxn)
 		next = boxes[bi + 1].LL.y > boxes[bi].LL.y ? 1 : -1;
 	    if (prev != next) {
 		if (next == -1 || prev == 1) {
@@ -405,14 +382,15 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	    } 
 	    else {
 		if (!(prev == -1 && next == -1)) {
-		    agerr(AGERR, "in routesplines, illegal values of prev %d and next %d, line %d\n", prev, next, __LINE__);
+		    free(polypoints);
+		    agerrorf("in routesplines, illegal values of prev %d and next %d, line %d\n", prev, next, __LINE__);
 		    return NULL;
 		}
 	    }
 	}
-	for (bi = boxn - 1; bi >= 0; bi--) {
+	for (bi = boxn - 1; bi != SIZE_MAX; bi--) {
 	    next = prev = 0;
-	    if (bi < boxn - 1)
+	    if (bi + 1 < boxn)
 		prev = boxes[bi].LL.y > boxes[bi + 1].LL.y ? -1 : 1;
 	    if (bi > 0)
 		next = boxes[bi - 1].LL.y > boxes[bi].LL.y ? 1 : -1;
@@ -438,7 +416,8 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	    else {
 		if (!(prev == -1 && next == -1)) {
 		    /* it went badly, e.g. degenerate box in boxlist */
-		    agerr(AGERR, "in routesplines, illegal values of prev %d and next %d, line %d\n", prev, next, __LINE__);
+		    free(polypoints);
+		    agerrorf("in routesplines, illegal values of prev %d and next %d, line %d\n", prev, next, __LINE__);
 		    return NULL; /* for correctness sake, it's best to just stop */
 		}
 		polypoints[pi].x = boxes[bi].UR.x;
@@ -453,23 +432,24 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	}
     }
     else {
-	agerr(AGERR, "in routesplines, edge is a loop at %s\n", agnameof(aghead(realedge)));
+	free(polypoints);
+	agerrorf("in routesplines, edge is a loop at %s\n", agnameof(aghead(realedge)));
 	return NULL;
     }
 
     if (flip) {
-	for (bi = 0; bi < boxn; bi++) {
+	for (size_t bi = 0; bi < boxn; bi++) {
 	    double v = boxes[bi].UR.y;
 	    boxes[bi].UR.y = -1*boxes[bi].LL.y;
 	    boxes[bi].LL.y = -v;
 	}
-	for (int i = 0; i < pi; i++)
+	for (size_t i = 0; i < pi; i++)
 	    polypoints[i].y *= -1;
     }
 
-    static const double INITIAL_LLX = INT_MAX;
-    static const double INITIAL_URX = INT_MIN;
-    for (bi = 0; bi < boxn; bi++) {
+    static const double INITIAL_LLX = DBL_MAX;
+    static const double INITIAL_URX = -DBL_MAX;
+    for (size_t bi = 0; bi < boxn; bi++) {
 	boxes[bi].LL.x = INITIAL_LLX;
 	boxes[bi].UR.x = INITIAL_URX;
     }
@@ -477,7 +457,8 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
     eps[0].x = pp->start.p.x, eps[0].y = pp->start.p.y;
     eps[1].x = pp->end.p.x, eps[1].y = pp->end.p.y;
     if (Pshortestpath(&poly, eps, &pl) < 0) {
-	agerr(AGERR, "in routesplines, Pshortestpath failed\n");
+	free(polypoints);
+	agerrorf("in routesplines, Pshortestpath failed\n");
 	return NULL;
     }
 #ifdef DEBUG
@@ -491,11 +472,8 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	make_polyline (pl, &spl);
     }
     else {
-	if (poly.pn > edgen) {
-	    edges = ALLOC(poly.pn, edges, Pedge_t);
-	    edgen = poly.pn;
-	}
-	for (edgei = 0; edgei < poly.pn; edgei++) {
+	Pedge_t *edges = gv_calloc(poly.pn, sizeof(Pedge_t));
+	for (size_t edgei = 0; edgei < poly.pn; edgei++) {
 	    edges[edgei].a = polypoints[edgei];
 	    edges[edgei].b = polypoints[(edgei + 1) % poly.pn];
 	}
@@ -511,9 +489,12 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	    evs[1].x = evs[1].y = 0;
 
 	if (Proutespline(edges, poly.pn, pl, evs, &spl) < 0) {
-	    agerr(AGERR, "in routesplines, Proutespline failed\n");
+	    free(edges);
+	    free(polypoints);
+	    agerrorf("in routesplines, Proutespline failed\n");
 	    return NULL;
 	}
+	free(edges);
 #ifdef DEBUG
 	if (debugleveln(realedge, 3)) {
 	    psprintspline(spl);
@@ -523,31 +504,32 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
     }
     pointf *ps = calloc(spl.pn, sizeof(ps[0]));
     if (ps == NULL) {
-	agerr(AGERR, "cannot allocate ps\n");
+	free(polypoints);
+	agerrorf("cannot allocate ps\n");
 	return NULL;  /* Bailout if no memory left */
     }
 
     unbounded = true;
-    for (splinepi = 0; splinepi < spl.pn; splinepi++) {
+    for (size_t splinepi = 0; splinepi < spl.pn; splinepi++) {
 	ps[splinepi] = spl.ps[splinepi];
     }
 
+    double delta = INIT_DELTA;
     for (loopcnt = 0; unbounded && loopcnt < LOOP_TRIES; loopcnt++) {
-	limitBoxes (boxes, boxn, ps, spl.pn, delta);
+	limitBoxes(boxes, boxn, ps, spl.pn, delta);
 
     /* The following check is necessary because if a box is not very 
      * high, it is possible that the sampling above might miss it.
      * Therefore, we make the sample finer until all boxes have
-     * valid values. cf. bug 456. Would making sp[] pointfs help?
+     * valid values. cf. bug 456.
      */
+	size_t bi;
 	for (bi = 0; bi < boxn; bi++) {
 	/* these fp equality tests are used only to detect if the
 	 * values have been changed since initialization - ok */
-	    if (memcmp(&boxes[bi].LL.x, &INITIAL_LLX, sizeof(boxes[bi].LL.x)) == 0 ||
-	        memcmp(&boxes[bi].UR.x, &INITIAL_URX, sizeof(boxes[bi].UR.x)) == 0) {
+	    if (is_exactly_equal(boxes[bi].LL.x, INITIAL_LLX) ||
+	        is_exactly_equal(boxes[bi].UR.x, INITIAL_URX)) {
 		delta *= 2; /* try again with a finer interval */
-		if (delta > INT_MAX/boxn) /* in limitBoxes, boxn*delta must fit in an int, so give up */
-		    loopcnt = LOOP_TRIES;
 		break;
 	    }
 	}
@@ -561,9 +543,9 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	 * loop and we can see the bad edge, and even use the showboxes scaffolding.
 	 */
 	Ppolyline_t polyspl;
-	agerr(AGWARN, "Unable to reclaim box space in spline routing for edge \"%s\" -> \"%s\". Something is probably seriously wrong.\n", agnameof(agtail(realedge)), agnameof(aghead(realedge)));
+	agwarningf("Unable to reclaim box space in spline routing for edge \"%s\" -> \"%s\". Something is probably seriously wrong.\n", agnameof(agtail(realedge)), agnameof(aghead(realedge)));
 	make_polyline (pl, &polyspl);
-	limitBoxes (boxes, boxn, polyspl.ps, polyspl.pn, INIT_DELTA);
+	limitBoxes(boxes, boxn, polyspl.ps, polyspl.pn, INIT_DELTA);
     }
 
     *npoints = spl.pn;
@@ -577,31 +559,35 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	printboxes(boxn, boxes);
 #endif
 
+    free(polypoints);
     return ps;
 }
 
-pointf *routesplines(path * pp, int *npoints)
-{
-    return _routesplines (pp, npoints, 0);
+pointf *routesplines(path *pp, size_t *npoints) {
+  return routesplines_(pp, npoints, 0);
 }
 
-pointf *routepolylines(path * pp, int *npoints)
-{
-    return _routesplines (pp, npoints, 1);
+pointf *routepolylines(path *pp, size_t *npoints) {
+  return routesplines_(pp, npoints, 1);
 }
 
-static int overlap(int i0, int i1, int j0, int j1)
-{
-    /* i'll bet there's an elegant way to do this */
-    if (i1 <= j0)
-	return 0;
-    if (i0 >= j1)
-	return 0;
-    if (j0 <= i0 && i0 <= j1)
-	return j1 - i0;
-    if (j0 <= i1 && i1 <= j1)
-	return i1 - j0;
-    return MIN(i1 - i0, j1 - j0);
+static double overlap(double i0, double i1, double j0, double j1) {
+  if (i1 <= j0)
+    return 0;
+  if (i0 >= j1)
+    return 0;
+
+  // does the first interval subsume the second?
+  if (i0 <= j0 && i1 >= j1)
+    return i1 - i0;
+  // does the second interval subsume the first?
+  if (j0 <= i0 && j1 >= i1)
+    return j1 - j0;
+
+  if (j0 <= i0 && i0 <= j1)
+    return j1 - i0;
+  assert(j0 <= i1 && i1 <= j1);
+  return i1 - j0;
 }
 
 
@@ -614,15 +600,13 @@ static int overlap(int i0, int i1, int j0, int j1)
  *
  * Return 1 on failure; 0 on success.
  */
-static int checkpath(int boxn, boxf* boxes, path* thepath)
-{
+static int checkpath(size_t boxn, boxf *boxes, path *thepath) {
     boxf *ba, *bb;
-    int bi, i, errs, l, r, d, u;
-    int xoverlap, yoverlap;
+    int errs, l, r, d, u;
 
     /* remove degenerate boxes. */
-    i = 0;
-    for (bi = 0; bi < boxn; bi++) {
+    size_t i = 0;
+    for (size_t bi = 0; bi < boxn; bi++) {
 	if (fabs(boxes[bi].LL.y - boxes[bi].UR.y) < .01)
 	    continue;
 	if (fabs(boxes[bi].LL.x - boxes[bi].UR.x) < .01)
@@ -634,14 +618,15 @@ static int checkpath(int boxn, boxf* boxes, path* thepath)
 
     ba = &boxes[0];
     if (ba->LL.x > ba->UR.x || ba->LL.y > ba->UR.y) {
-	agerr(AGERR, "in checkpath, box 0 has LL coord > UR coord\n");
+	agerrorf("in checkpath, box 0 has LL coord > UR coord\n");
 	printpath(thepath);
 	return 1;
     }
-    for (bi = 0; bi < boxn - 1; bi++) {
+    for (size_t bi = 0; bi + 1 < boxn; bi++) {
 	ba = &boxes[bi], bb = &boxes[bi + 1];
 	if (bb->LL.x > bb->UR.x || bb->LL.y > bb->UR.y) {
-	    agerr(AGERR, "in checkpath, box %d has LL coord > UR coord\n", bi + 1);
+	    agerrorf("in checkpath, box %" PRISIZE_T " has LL coord > UR coord\n",
+	             bi + 1);
 	    printpath(thepath);
 	    return 1;
 	}
@@ -651,8 +636,8 @@ static int checkpath(int boxn, boxf* boxes, path* thepath)
 	u = ba->LL.y > bb->UR.y ? 1 : 0;
 	errs = l + r + d + u;
 	if (errs > 0 && Verbose) {
-	    fprintf(stderr, "in checkpath, boxes %d and %d don't touch\n",
-		    bi, bi + 1);
+	    fprintf(stderr, "in checkpath, boxes %" PRISIZE_T " and %" PRISIZE_T
+	            " don't touch\n", bi, bi + 1);
 	    printpath(thepath);
 	}
 	if (errs > 0) {
@@ -666,7 +651,7 @@ static int checkpath(int boxn, boxf* boxes, path* thepath)
 		xy = ba->UR.y, ba->UR.y = bb->LL.y, bb->LL.y = xy, d = 0;
 	    else if (u == 1)
 		xy = ba->LL.y, ba->LL.y = bb->UR.y, bb->UR.y = xy, u = 0;
-	    for (i = 0; i < errs - 1; i++) {
+	    for (int j = 0; j < errs - 1; j++) {
 		if (l == 1)
 		    xy = (ba->UR.x + bb->LL.x) / 2.0 + 0.5, ba->UR.x =
 			bb->LL.x = xy, l = 0;
@@ -682,9 +667,9 @@ static int checkpath(int boxn, boxf* boxes, path* thepath)
 	    }
 	}
 	/* check for overlapping boxes */
-	xoverlap = overlap(ba->LL.x, ba->UR.x, bb->LL.x, bb->UR.x);
-	yoverlap = overlap(ba->LL.y, ba->UR.y, bb->LL.y, bb->UR.y);
-	if (xoverlap && yoverlap) {
+	double xoverlap = overlap(ba->LL.x, ba->UR.x, bb->LL.x, bb->UR.x);
+	double yoverlap = overlap(ba->LL.y, ba->UR.y, bb->LL.y, bb->UR.y);
+	if (xoverlap > 0 && yoverlap > 0) {
 	    if (xoverlap < yoverlap) {
 		if (ba->UR.x - ba->LL.x > bb->UR.x - bb->LL.x) {
 		    /* take space from ba */
@@ -721,10 +706,6 @@ static int checkpath(int boxn, boxf* boxes, path* thepath)
 	|| thepath->start.p.x > boxes[0].UR.x
 	|| thepath->start.p.y < boxes[0].LL.y
 	|| thepath->start.p.y > boxes[0].UR.y) {
-	if (Verbose) {
-	    fprintf(stderr, "in checkpath, start port not in first box\n");
-	    printpath(thepath);
-	}
 	thepath->start.p.x = fmax(thepath->start.p.x, boxes[0].LL.x);
 	thepath->start.p.x = fmin(thepath->start.p.x, boxes[0].UR.x);
 	thepath->start.p.y = fmax(thepath->start.p.y, boxes[0].LL.y);
@@ -734,10 +715,6 @@ static int checkpath(int boxn, boxf* boxes, path* thepath)
 	|| thepath->end.p.x > boxes[boxn - 1].UR.x
 	|| thepath->end.p.y < boxes[boxn - 1].LL.y
 	|| thepath->end.p.y > boxes[boxn - 1].UR.y) {
-	if (Verbose) {
-	    fprintf(stderr, "in checkpath, end port not in last box\n");
-	    printpath(thepath);
-	}
 	thepath->end.p.x = fmax(thepath->end.p.x, boxes[boxn - 1].LL.x);
 	thepath->end.p.x = fmin(thepath->end.p.x, boxes[boxn - 1].UR.x);
 	thepath->end.p.y = fmax(thepath->end.p.y, boxes[boxn - 1].LL.y);
@@ -748,11 +725,9 @@ static int checkpath(int boxn, boxf* boxes, path* thepath)
 
 static void printpath(path * pp)
 {
-    int bi;
-
-    fprintf(stderr, "%d boxes:\n", pp->nbox);
-    for (bi = 0; bi < pp->nbox; bi++)
-	fprintf(stderr, "%d (%.5g, %.5g), (%.5g, %.5g)\n", bi,
+    fprintf(stderr, "%" PRISIZE_T " boxes:\n", pp->nbox);
+    for (size_t bi = 0; bi < pp->nbox; bi++)
+	fprintf(stderr, "%" PRISIZE_T " (%.5g, %.5g), (%.5g, %.5g)\n", bi,
 		pp->boxes[bi].LL.x, pp->boxes[bi].LL.y,
 	       	pp->boxes[bi].UR.x, pp->boxes[bi].UR.y);
     fprintf(stderr, "start port: (%.5g, %.5g), tangent angle: %.5g, %s\n",
@@ -772,102 +747,26 @@ static pointf get_centroid(Agraph_t *g)
     return sum;
 }
 
-//generic vector structure
-typedef struct _tag_vec
-{
-    void** _mem;
-    size_t _elems;
-    size_t _capelems;
-} vec;
+DEFINE_LIST(nodes, node_t *)
 
-static vec* vec_new(void)
-{
-    vec* pvec = malloc(sizeof(vec));
-    pvec->_capelems = 10;
-    pvec->_elems = 0;
-    pvec->_mem = malloc(pvec->_capelems * sizeof(void*));
-    return pvec;
-}
-
-static size_t vec_length(const vec* pvec)
-{
-    return pvec->_elems;
-}
-
-static void* vec_get(vec* pvec, size_t index)
-{
-    assert(index < pvec->_elems);
-    return pvec->_mem[index];
-}
-
-static void vec_delete(vec* pvec)
-{
-    free(pvec->_mem);
-    free(pvec);
-}
-
-// cycles is assumed to be a vec of vec of nodes.
-static void cycles_delete(vec* cycles) {
-  for (size_t i = 0; i < vec_length(cycles); ++i) {
-    vec_delete(vec_get(cycles, i));
+static void nodes_delete(nodes_t *pvec) {
+  if (pvec != NULL) {
+    nodes_free(pvec);
   }
-  vec_delete(cycles);
+  free(pvec);
 }
 
-static void vec_push_back(vec* pvec, void* data)
-{
-    if (pvec->_elems == pvec->_capelems) {
-		pvec->_capelems += 10;
-		pvec->_mem = realloc(pvec->_mem, pvec->_capelems * sizeof(void*));
-	}
-    pvec->_mem[pvec->_elems++] = data;  
-}
+DEFINE_LIST_WITH_DTOR(cycles, nodes_t *, nodes_delete)
 
-static void* vec_pop(vec* pvec)
-{
-	if (pvec->_elems > 0)
-		return pvec->_mem[--pvec->_elems];
-	return NULL;
-}
-
-static bool vec_contains(vec* pvec, void* item)
-{
-	for (size_t i=0; i < pvec->_elems; ++i) {
-		if (pvec->_mem[i] == item)
-			return true;
-	}
-
-	return false;
-}
-
-static vec* vec_copy(vec* pvec)
-{
-    vec* nvec = malloc(sizeof(vec));
-    nvec->_capelems = pvec->_capelems;
-    nvec->_elems = pvec->_elems;
-    nvec->_mem = malloc(pvec->_capelems * sizeof(void*));
-	memcpy(nvec->_mem, pvec->_mem, pvec->_elems * sizeof(void*));
-    return nvec;
-}
-//end generic vector structure
-
-static bool cycle_contains_edge(vec* cycle, edge_t* edge)
-{
+static bool cycle_contains_edge(nodes_t *cycle, edge_t *edge) {
 	node_t* start = agtail(edge);
 	node_t* end = aghead(edge);
-	node_t* c_start;
-	node_t* c_end;
 
-	size_t cycle_len = vec_length(cycle);
+	const size_t cycle_len = nodes_size(cycle);
 
 	for (size_t i=0; i < cycle_len; ++i) {
-		if (i == 0) {
-			c_start = vec_get(cycle, cycle_len-1);
-		} else {
-			c_start = vec_get(cycle, i-1);
-		}
-	
-		c_end = vec_get(cycle, i);
+		const node_t *c_start = nodes_get(cycle, i == 0 ? cycle_len - 1 : i - 1);
+		const node_t *c_end = nodes_get(cycle, i);
 
 		if (c_start == start && c_end == end)
 			return true;
@@ -877,28 +776,24 @@ static bool cycle_contains_edge(vec* cycle, edge_t* edge)
 	return false;
 }
 
-static bool is_cycle_unique(vec* cycles, vec* cycle)
-{
-	size_t cycle_len = vec_length(cycle);
-	size_t n_cycles = vec_length(cycles);
-	size_t c; //cycles counter
+static bool eq(const node_t *a, const node_t *b) { return a == b; }
+
+static bool is_cycle_unique(cycles_t *cycles, nodes_t *cycle) {
+	const size_t cycle_len = nodes_size(cycle);
 	size_t i; //node counter
 
-	vec* cur_cycle;
-	size_t cur_cycle_len;
-	void* cur_cycle_item;
 	bool all_items_match;
 
-	for (c=0; c < n_cycles; ++c) {
-		cur_cycle = vec_get(cycles, c);
-		cur_cycle_len = vec_length(cur_cycle);
+	for (size_t c = 0; c < cycles_size(cycles); ++c) {
+		nodes_t *cur_cycle = cycles_get(cycles, c);
+		const size_t cur_cycle_len = nodes_size(cur_cycle);
 
 		//if all the items match in equal length cycles then we're not unique
 		if (cur_cycle_len == cycle_len) {
 			all_items_match = true;
 			for (i=0; i < cur_cycle_len; ++i) {
-				cur_cycle_item = vec_get(cur_cycle, i);
-				if (!vec_contains(cycle, cur_cycle_item)) {
+				node_t *cur_cycle_item = nodes_get(cur_cycle, i);
+				if (!nodes_contains(cycle, cur_cycle_item, eq)) {
 					all_items_match = false;
 					break;
 				}
@@ -911,64 +806,63 @@ static bool is_cycle_unique(vec* cycles, vec* cycle)
 	return true;
 }
 
-static void dfs(graph_t *g, node_t* search, vec* visited, node_t* end, vec* cycles)
-{
+static void dfs(graph_t *g, node_t *search, nodes_t *visited, node_t *end,
+                cycles_t *cycles) {
 	edge_t* e;
 	node_t* n;
 
-	if (vec_contains(visited, search)) {
+	if (nodes_contains(visited, search, eq)) {
 		if (search == end) {
 			if (is_cycle_unique(cycles, visited)) {
-				vec* cycle = vec_copy(visited);
-				vec_push_back(cycles, cycle);
+				nodes_t *cycle = gv_alloc(sizeof(nodes_t));
+				*cycle = nodes_copy(visited);
+				cycles_append(cycles, cycle);
 			}
 		}
 	} else {
-		vec_push_back(visited, search);
+		nodes_append(visited, search);
 		for (e = agfstout(g, search); e; e = agnxtout(g, e)) {
 			n = aghead(e);
 			dfs(g, n, visited, end, cycles);
 		}
-		vec_pop(visited);
+		if (!nodes_is_empty(visited)) {
+			(void)nodes_pop_back(visited);
+		}
 	}
 }
 
-// Returns a vec of vec of nodes (aka a vector of cycles), which must be freed using cycles_delete.
-static vec* find_all_cycles(graph_t *g)
-{
+// Returns a vec of vec of nodes (aka a vector of cycles)
+static cycles_t find_all_cycles(graph_t *g) {
     node_t *n;
 
-    vec* alloced_cycles = vec_new(); //vector of vectors of nodes -- AKA cycles to delete
-    vec* cycles = vec_new(); //vector of vectors of nodes AKA a vector of cycles
-    vec* cycle;
+    // vector of vectors of nodes -- AKA cycles to delete
+    cycles_t alloced_cycles = {0};
+    cycles_t cycles = {0}; // vector of vectors of nodes AKA a vector of cycles
 
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-		cycle = vec_new();
-		vec_push_back(alloced_cycles, cycle); //keep track of all items we allocate to clean up at the end of this function
+		nodes_t *cycle = gv_alloc(sizeof(nodes_t));
+		// keep track of all items we allocate to clean up at the end of this function
+		cycles_append(&alloced_cycles, cycle);
 		
-		dfs(g, n, cycle, n, cycles);
+		dfs(g, n, cycle, n, &cycles);
 	}
 	
-	cycles_delete(alloced_cycles); //cycles contains copied vecs
+	cycles_free(&alloced_cycles); // cycles contains copied vecs
     return cycles;
 }
 
-static vec* find_shortest_cycle_with_edge(vec* cycles, edge_t* edge, size_t min_size)
-{
-	size_t c; //cycle counter
-	size_t cycles_len = vec_length(cycles);
-	vec* cycle;
-	size_t cycle_len; 
-	vec* shortest = 0;
+static nodes_t *find_shortest_cycle_with_edge(cycles_t *cycles, edge_t *edge,
+                                              size_t min_size) {
+	nodes_t *shortest = NULL;
 
-	for (c=0; c < cycles_len; ++c) {
-		cycle = vec_get(cycles, c);
-		cycle_len = vec_length(cycle);
+	for (size_t c = 0; c < cycles_size(cycles); ++c) {
+		nodes_t *cycle = cycles_get(cycles, c);
+		size_t cycle_len = nodes_size(cycle);
 
 		if (cycle_len < min_size)
 			continue;
 
-		if (!shortest || vec_length(shortest) > cycle_len) {
+		if (shortest == NULL || nodes_size(shortest) > cycle_len) {
 			if (cycle_contains_edge(cycle, edge)) {
 				shortest = cycle;
 			}
@@ -979,34 +873,27 @@ static vec* find_shortest_cycle_with_edge(vec* cycles, edge_t* edge, size_t min_
 
 static pointf get_cycle_centroid(graph_t *g, edge_t* edge)
 {
-	vec* cycles = find_all_cycles(g);
+	cycles_t cycles = find_all_cycles(g);
 
 	//find the center of the shortest cycle containing this edge
 	//cycles of length 2 do their own thing, we want 3 or
-	vec* cycle = find_shortest_cycle_with_edge(cycles, edge, 3);
-	size_t cycle_len;
+	nodes_t *cycle = find_shortest_cycle_with_edge(&cycles, edge, 3);
     pointf sum = {0.0, 0.0};
-	size_t idx; //edge index
-	node_t *n;
 
 	if (cycle == NULL) {
-		if (cycles != NULL)
-			cycles_delete(cycles);
+		cycles_free(&cycles);
 		return get_centroid(g);
 	}
 
-	cycle_len = vec_length(cycle);
-
 	double cnt = 0;
-	for (idx=0; idx < cycle_len; ++idx) {
-		n = vec_get(cycle, idx);
+	for (size_t idx = 0; idx < nodes_size(cycle); ++idx) {
+		node_t *n = nodes_get(cycle, idx);
 		sum.x += ND_coord(n).x;
         sum.y += ND_coord(n).y;
         cnt++;
 	}
 
-	if (cycles != NULL)
-		cycles_delete(cycles);
+	cycles_free(&cycles);
 
 	sum.x /= cnt;
     sum.y /= cnt;
@@ -1034,10 +921,7 @@ static void bend(pointf spl[4], pointf centroid)
     spl[1].y = spl[2].y = a.y;
 }
 
-/* makeStraightEdge:
- *
- * FIX: handle ports on boundary?
- */
+// FIX: handle ports on boundary?
 void 
 makeStraightEdge(graph_t * g, edge_t * e, int et, splineInfo* sinfo)
 {
@@ -1047,20 +931,19 @@ makeStraightEdge(graph_t * g, edge_t * e, int et, splineInfo* sinfo)
     e0 = e;
     while (e0 != ED_to_virt(e0) && (e0 = ED_to_virt(e0))) e_cnt++;
 
-    edge_t **edge_list = N_NEW(e_cnt, edge_t*);
+    edge_t **edge_list = gv_calloc(e_cnt, sizeof(edge_t *));
     e0 = e;
     for (size_t i = 0; i < e_cnt; i++) {
 	edge_list[i] = e0;
 	e0 = ED_to_virt(e0);
     }
     assert(e_cnt <= INT_MAX);
-    makeStraightEdges(g, edge_list, (int)e_cnt, et, sinfo);
+    makeStraightEdges(g, edge_list, e_cnt, et, sinfo);
     free(edge_list);
 }
 
-void 
-makeStraightEdges(graph_t *g, edge_t **edge_list, int e_cnt, int et,
-                  splineInfo *sinfo) {
+void makeStraightEdges(graph_t *g, edge_t **edge_list, size_t e_cnt, int et,
+                       splineInfo *sinfo) {
     pointf dumb[4];
     bool curved = et == EDGETYPE_CURVED;
     pointf del;
@@ -1077,7 +960,6 @@ makeStraightEdges(graph_t *g, edge_t **edge_list, int e_cnt, int et,
 	return;
     }
 
-    edge_t *e0 = e;
     if (APPROXEQPT(dumb[0], dumb[3], MILLIPOINT)) {
 	/* degenerate case */
 	dumb[1] = dumb[0];
@@ -1092,7 +974,8 @@ makeStraightEdges(graph_t *g, edge_t **edge_list, int e_cnt, int et,
         };
 	double l_perp = hypot(perp.x, perp.y);
 	int xstep = GD_nodesep(g->root);
-	int dx = xstep * (e_cnt - 1) / 2;
+	assert(e_cnt - 1 <= INT_MAX);
+	int dx = xstep * (int)(e_cnt - 1) / 2;
 	dumb[1].x = dumb[0].x + dx * perp.x / l_perp;
 	dumb[1].y = dumb[0].y + dx * perp.y / l_perp;
 	dumb[2].x = dumb[3].x + dx * perp.x / l_perp;
@@ -1101,8 +984,8 @@ makeStraightEdges(graph_t *g, edge_t **edge_list, int e_cnt, int et,
 	del.y = -xstep * perp.y / l_perp;
     }
 
-    for (int i = 0; i < e_cnt; i++) {
-	e0 = edge_list[i];
+    for (size_t i = 0; i < e_cnt; i++) {
+	edge_t *e0 = edge_list[i];
 	pointf dumber[4];
 	if (aghead(e0) == head) {
 	    for (size_t j = 0; j < 4; j++) {
@@ -1117,7 +1000,7 @@ makeStraightEdges(graph_t *g, edge_t **edge_list, int e_cnt, int et,
 	    Ppoint_t pts[] = {dumber[0], dumber[1], dumber[2], dumber[3]};
 	    Ppolyline_t spl, line = {.pn = 4, .ps = pts};
 	    make_polyline (line, &spl);
-	    clip_and_install(e0, aghead(e0), spl.ps, spl.pn, sinfo);
+	    clip_and_install(e0, aghead(e0), spl.ps, (size_t)spl.pn, sinfo);
 	}
 	else
 	    clip_and_install(e0, aghead(e0), dumber, 4, sinfo);
