@@ -1,68 +1,36 @@
+/// @file
+/// @ingroup common_utils
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include <common/render.h>
-#include <cgraph/alloc.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/strview.h>
-#include <cgraph/tokenize.h>
+#include <common/geomprocs.h>
 #include <common/htmltable.h>
 #include <common/entities.h>
 #include <limits.h>
 #include <math.h>
 #include <gvc/gvc.h>
-#include <cgraph/strcasecmp.h>
+#include <stdatomic.h>
 #include <stddef.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <unistd.h>
-
-#include <ctype.h>
-
-/*
- *  a queue of nodes
- */
-nodequeue *new_queue(size_t sz) {
-    nodequeue *q = gv_alloc(sizeof(nodequeue));
-
-    if (sz <= 1)
-	sz = 2;
-    q->head = q->tail = q->store = gv_calloc(sz, sizeof(node_t*));
-    q->limit = q->store + sz;
-    return q;
-}
-
-void free_queue(nodequeue * q)
-{
-    free(q->store);
-    free(q);
-}
-
-void enqueue(nodequeue * q, node_t * n)
-{
-    *q->tail++ = n;
-    if (q->tail >= q->limit)
-	q->tail = q->store;
-}
-
-node_t *dequeue(nodequeue * q)
-{
-    node_t *n;
-    if (q->head == q->tail)
-	n = NULL;
-    else {
-	n = *q->head++;
-	if (q->head >= q->limit)
-	    q->head = q->store;
-    }
-    return n;
-}
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/gv_ctype.h>
+#include <util/gv_math.h>
+#include <util/startswith.h>
+#include <util/strcasecmp.h>
+#include <util/streq.h>
+#include <util/strview.h>
+#include <util/tokenize.h>
 
 int late_int(void *obj, attrsym_t *attr, int defaultValue, int minimum) {
     if (attr == NULL)
@@ -76,7 +44,7 @@ int late_int(void *obj, attrsym_t *attr, int defaultValue, int minimum) {
         return defaultValue; /* invalid int format */
     if (rv < minimum)
         return minimum;
-    else return (int)rv;
+    return (int)rv;
 }
 
 double late_double(void *obj, attrsym_t *attr, double defaultValue,
@@ -92,11 +60,10 @@ double late_double(void *obj, attrsym_t *attr, double defaultValue,
         return defaultValue; /* invalid double format */
     if (rv < minimum)
         return minimum;
-    else return rv;
+    return rv;
 }
 
-/* get_inputscale:
- * Return value for PSinputscale. If this is > 0, it has been set on the
+/** Return value for PSinputscale. If this is > 0, it has been set on the
  * command line and this value is used.
  * Otherwise, we check the graph's inputscale attribute. If this is not set
  * or has a bad value, we return -1.
@@ -106,8 +73,8 @@ double late_double(void *obj, attrsym_t *attr, double defaultValue,
 double get_inputscale(graph_t *g) {
     if (PSinputscale > 0) return PSinputscale;  /* command line flag prevails */
     double d = late_double(g, agfindgraphattr(g, "inputscale"), -1, 0);
-    if (d == 0) return POINTS_PER_INCH;
-    else return d;
+    if (is_exactly_zero(d)) return POINTS_PER_INCH;
+    return d;
 }
 
 char *late_string(void *obj, attrsym_t *attr, char *defaultValue) {
@@ -130,7 +97,6 @@ bool late_bool(void *obj, attrsym_t *attr, bool defaultValue) {
     return mapbool(agxget(obj, attr));
 }
 
-/* union-find */
 node_t *UF_find(node_t * n)
 {
     while (ND_UF_parent(n) && ND_UF_parent(n) != n) {
@@ -196,14 +162,13 @@ pointf coord(node_t * n)
 #define W_DEGREE 5
 
 /*
- *  Bezier :
  *	Evaluate a Bezier curve at a particular parameter value
  *      Fill in control points for resulting sub-curves if "Left" and
  *	"Right" are non-null.
  *
  */
-pointf Bezier(pointf * V, int degree, double t, pointf * Left, pointf * Right)
-{
+pointf Bezier(pointf *V, double t, pointf *Left, pointf *Right) {
+    const int degree = 3;
     int i, j;			/* Index variables      */
     pointf Vtemp[W_DEGREE + 1][W_DEGREE + 1];
 
@@ -240,8 +205,7 @@ edge_t *debug_getedge(graph_t * g, char *s0, char *s1)
     n1 = agfindnode(g, s1);
     if (n0 && n1)
 	return agfindedge(g, n0, n1);
-    else
-	return NULL;
+    return NULL;
 }
 Agraphinfo_t* GD_info(graph_t * g) { return ((Agraphinfo_t*)AGDATA(g));}
 Agnodeinfo_t* ND_info(node_t * n) { return ((Agnodeinfo_t*)AGDATA(n));}
@@ -304,7 +268,7 @@ const char *safefile(const char *filename)
 
     if (HTTPServerEnVar) {   /* If used as a server */
 	if (onetime) {
-	    agerr(AGWARN,
+	    agwarningf(
 		      "file loading is disabled because the environment contains SERVER_NAME=\"%s\"\n",
 		      HTTPServerEnVar);
 	    onetime = false;
@@ -312,6 +276,24 @@ const char *safefile(const char *filename)
 	return NULL;
     }
 
+    if (Gvfilepath != NULL) {
+	if (pathlist == NULL) {
+	    free(dirs);
+	    pathlist = Gvfilepath;
+	    dirs = mkDirlist(pathlist);
+	}
+
+	const char *str = filename;
+	for (const char *sep = "/\\:"; *sep != '\0'; ++sep) {
+	    const char *p = strrchr(str, *sep);
+	    if (p != NULL) {
+	        str = ++p;
+	    }
+	}
+
+	return findPath(dirs, str);
+    }
+
     if (pathlist != Gvimagepath) {
 	free (dirs);
 	dirs = NULL;
@@ -347,10 +329,9 @@ bool mapBool(const char *p, bool defaultValue) {
 	return true;
     if (!strcasecmp(p, "yes"))
 	return true;
-    if (isdigit((int)*p))
+    if (gv_isdigit(*p))
 	return atoi(p) != 0;
-    else
-        return defaultValue;
+    return defaultValue;
 }
 
 bool mapbool(const char *p)
@@ -360,23 +341,23 @@ bool mapbool(const char *p)
 
 pointf dotneato_closest(splines * spl, pointf pt)
 {
-    int i, j, k, besti, bestj;
     double bestdist2, d2, dlow2, dhigh2; /* squares of distances */
     double low, high, t;
     pointf c[4], pt2;
     bezier bz;
 
-    besti = bestj = -1;
+    size_t besti = SIZE_MAX;
+    size_t bestj = SIZE_MAX;
     bestdist2 = 1e+38;
-    for (i = 0; i < spl->size; i++) {
+    for (size_t i = 0; i < spl->size; i++) {
 	bz = spl->list[i];
-	for (j = 0; j < bz.size; j++) {
+	for (size_t j = 0; j < bz.size; j++) {
 	    pointf b;
 
 	    b.x = bz.list[j].x;
 	    b.y = bz.list[j].y;
 	    d2 = DIST2(b, pt);
-	    if (bestj == -1 || d2 < bestdist2) {
+	    if (bestj == SIZE_MAX || d2 < bestdist2) {
 		besti = i;
 		bestj = j;
 		bestdist2 = d2;
@@ -391,8 +372,8 @@ pointf dotneato_closest(splines * spl, pointf pt)
      */
     if (bestj == bz.size-1)
 	bestj--;
-    j = 3*(bestj / 3);
-    for (k = 0; k < 4; k++) {
+    const size_t j = 3 * (bestj / 3);
+    for (size_t k = 0; k < 4; k++) {
 	c[k].x = bz.list[j + k].x;
 	c[k].y = bz.list[j + k].y;
     }
@@ -402,7 +383,7 @@ pointf dotneato_closest(splines * spl, pointf pt)
     dhigh2 = DIST2(c[3], pt);
     do {
 	t = (low + high) / 2.0;
-	pt2 = Bezier(c, 3, t, NULL, NULL);
+	pt2 = Bezier(c, t, NULL, NULL);
 	if (fabs(dlow2 - dhigh2) < 1.0)
 	    break;
 	if (fabs(high - low) < .00001)
@@ -418,62 +399,6 @@ pointf dotneato_closest(splines * spl, pointf pt)
     return pt2;
 }
 
-pointf spline_at_y(splines * spl, double y)
-{
-    int i, j;
-    double low, high, d, t;
-    pointf c[4], p;
-    static bezier bz;
-
-/* this caching seems to prevent p.x from getting set from bz.list[0].x
-	- optimizer problem ? */
-
-	for (i = 0; i < spl->size; i++) {
-	    bz = spl->list[i];
-	    if (BETWEEN(bz.list[bz.size - 1].y, y, bz.list[0].y))
-		break;
-	}
-    if (y > bz.list[0].y)
-	p = bz.list[0];
-    else if (y < bz.list[bz.size - 1].y)
-	p = bz.list[bz.size - 1];
-    else {
-	for (i = 0; i < bz.size; i += 3) {
-	    for (j = 0; j < 3; j++) {
-		if (bz.list[i + j].y <= y && y <= bz.list[i + j + 1].y)
-		    break;
-		if (bz.list[i + j].y >= y && y >= bz.list[i + j + 1].y)
-		    break;
-	    }
-	    if (j < 3)
-		break;
-	}
-	assert(i < bz.size);
-	for (j = 0; j < 4; j++) {
-	    c[j].x = bz.list[i + j].x;
-	    c[j].y = bz.list[i + j].y;
-	    /* make the spline be monotonic in Y, awful but it works for now */
-	    if (j > 0 && c[j].y > c[j - 1].y)
-		c[j].y = c[j - 1].y;
-	}
-	low = 0.0;
-	high = 1.0;
-	do {
-	    t = (low + high) / 2.0;
-	    p = Bezier(c, 3, t, NULL, NULL);
-	    d = p.y - y;
-	    if (fabs(d) <= 1)
-		break;
-	    if (d < 0)
-		high = t;
-	    else
-		low = t;
-	} while (1);
-    }
-    p.y = y;
-    return p;
-}
-
 static int Tflag;
 void gvToggle(int s)
 {
@@ -510,7 +435,7 @@ void common_init_node(node_t * n)
     fi.fontname = late_nnstring(n, N_fontname, DEFAULT_FONTNAME);
     fi.fontcolor = late_nnstring(n, N_fontcolor, DEFAULT_COLOR);
     ND_label(n) = make_label(n, str,
-	        (aghtmlstr(str) ? LT_HTML : LT_NONE) | ( (shapeOf(n) == SH_RECORD) ? LT_RECD : LT_NONE),
+	        (aghtmlstr(str) ? LT_HTML : LT_NONE) | (shapeOf(n) == SH_RECORD ? LT_RECD : LT_NONE),
 		fi.fontsize, fi.fontname, fi.fontcolor);
     if (N_xlabel && (str = agxget(n, N_xlabel)) && str[0]) {
 	ND_xlabel(n) = make_label(n, str, aghtmlstr(str) ? LT_HTML : LT_NONE,
@@ -518,7 +443,10 @@ void common_init_node(node_t * n)
 	GD_has_labels(agraphof(n)) |= NODE_XLABEL;
     }
 
-    ND_showboxes(n) = late_int(n, N_showboxes, 0, 0);
+    {
+	const int showboxes = imin(late_int(n, N_showboxes, 0, 0), UCHAR_MAX);
+	ND_showboxes(n) = (unsigned char)showboxes;
+    }
     ND_shape(n)->fns->initfn(n);
 }
 
@@ -539,10 +467,7 @@ initFontLabelEdgeAttr(edge_t * e, struct fontinfo *fi,
     lfi->fontcolor = late_nnstring(e, E_labelfontcolor, fi->fontcolor);
 }
 
-/* noClip:
- * Return true if head/tail end of edge should not be clipped
- * to node.
- */
+/// Return true if head/tail end of edge should not be clipped to node.
 static bool
 noClip(edge_t *e, attrsym_t* sym)
 {
@@ -557,8 +482,6 @@ noClip(edge_t *e, attrsym_t* sym)
     return rv;
 }
 
-/*chkPort:
- */
 static port
 chkPort (port (*pf)(node_t*, char*, char*), node_t* n, char* s)
 {
@@ -580,10 +503,8 @@ chkPort (port (*pf)(node_t*, char*, char*), node_t* n, char* s)
 }
 
 /* return true if edge has label */
-int common_init_edge(edge_t * e)
-{
+void common_init_edge(edge_t *e) {
     char *str;
-    int r = 0;
     struct fontinfo fi;
     struct fontinfo lfi;
     graph_t *sg = agraphof(agtail(e));
@@ -591,13 +512,11 @@ int common_init_edge(edge_t * e)
     fi.fontname = NULL;
     lfi.fontname = NULL;
     if (E_label && (str = agxget(e, E_label)) && str[0]) {
-	r = 1;
 	initFontEdgeAttr(e, &fi);
 	ED_label(e) = make_label(e, str, aghtmlstr(str) ? LT_HTML : LT_NONE,
 				fi.fontsize, fi.fontname, fi.fontcolor);
 	GD_has_labels(sg) |= EDGE_LABEL;
-	ED_label_ontop(e) =
-	    mapbool(late_string(e, E_label_float, "false")) ? TRUE : FALSE;
+	ED_label_ontop(e) = mapbool(late_string(e, E_label_float, "false"));
     }
 
     if (E_xlabel && (str = agxget(e, E_xlabel)) && str[0]) {
@@ -642,12 +561,8 @@ int common_init_edge(edge_t * e)
     ED_head_port(e) = chkPort(ND_shape(aghead(e))->fns->portfn, aghead(e), str);
     if (noClip(e, E_headclip))
 	ED_head_port(e).clip = false;
-
-    return r;
 }
 
-/* addLabelBB:
- */
 static boxf addLabelBB(boxf bb, textlabel_t * lp, bool flipxy)
 {
     double width, height;
@@ -679,20 +594,19 @@ static boxf addLabelBB(boxf bb, textlabel_t * lp, bool flipxy)
     return bb;
 }
 
-/* polyBB:
- * Compute the bounding box of a polygon.
+/** Compute the bounding box of a polygon.
  * We only need to use the outer periphery.
  */
 boxf
 polyBB (polygon_t* poly)
 {
-    int i, sides = poly->sides;
-    int peris = MAX(poly->peripheries,1);
+    const size_t sides = poly->sides;
+    const size_t peris = MAX(poly->peripheries, (size_t)1);
     pointf* verts = poly->vertices + (peris-1)*sides;
     boxf bb;
 
     bb.LL = bb.UR = verts[0];
-    for (i = 1; i < sides; i++) {
+    for (size_t i = 1; i < sides; i++) {
 	bb.LL.x = MIN(bb.LL.x,verts[i].x);
 	bb.LL.y = MIN(bb.LL.y,verts[i].y);
 	bb.UR.x = MAX(bb.UR.x,verts[i].x);
@@ -701,8 +615,7 @@ polyBB (polygon_t* poly)
     return bb;
 }
 
-/* updateBB:
- * Reset graph's bounding box to include bounding box of the given label.
+/** Reset graph's bounding box to include bounding box of the given label.
  * Assume the label's position has been set.
  */
 void updateBB(graph_t * g, textlabel_t * lp)
@@ -710,8 +623,7 @@ void updateBB(graph_t * g, textlabel_t * lp)
     GD_bb(g) = addLabelBB(GD_bb(g), lp, GD_flip(g));
 }
 
-/* compute_bb:
- * Compute bounding box of g using nodes, splines, and clusters.
+/** Compute bounding box of g using nodes, splines, and clusters.
  * Assumes bb of clusters already computed.
  * store in GD_bb.
  */
@@ -722,7 +634,6 @@ void compute_bb(graph_t * g)
     boxf b, bb;
     boxf BF;
     pointf ptf, s2;
-    int i, j;
 
     if (agnnodes(g) == 0 && GD_n_cluster(g) == 0) {
 	bb.LL = (pointf){0};
@@ -739,17 +650,17 @@ void compute_bb(graph_t * g)
 	b.LL = sub_pointf(ptf, s2);
 	b.UR = add_pointf(ptf, s2);
 
-	EXPANDBB(bb,b);
+	EXPANDBB(&bb, b);
 	if (ND_xlabel(n) && ND_xlabel(n)->set) {
 	    bb = addLabelBB(bb, ND_xlabel(n), GD_flip(g));
 	}
 	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
 	    if (ED_spl(e) == 0)
 		continue;
-	    for (i = 0; i < ED_spl(e)->size; i++) {
-		for (j = 0; j < (((Agedgeinfo_t*)AGDATA(e))->spl)->list[i].size; j++) {
+	    for (size_t i = 0; i < ED_spl(e)->size; i++) {
+		for (size_t j = 0; j < (((Agedgeinfo_t*)AGDATA(e))->spl)->list[i].size; j++) {
 		    ptf = ED_spl(e)->list[i].list[j];
-		    EXPANDBP(bb,ptf);
+		    expandbp(&bb, ptf);
 		}
 	    }
 	    if (ED_label(e) && ED_label(e)->set) {
@@ -767,9 +678,9 @@ void compute_bb(graph_t * g)
 	}
     }
 
-    for (i = 1; i <= GD_n_cluster(g); i++) {
+    for (int i = 1; i <= GD_n_cluster(g); i++) {
 	B2BF(GD_bb(GD_clust(g)[i]), BF);
-	EXPANDBB(bb,BF);
+	EXPANDBB(&bb, BF);
     }
     if (GD_label(g) && GD_label(g)->set) {
 	bb = addLabelBB(bb, GD_label(g), GD_flip(g));
@@ -781,11 +692,10 @@ void compute_bb(graph_t * g)
 bool is_a_cluster (Agraph_t* g)
 {
   return g == g->root || !strncasecmp(agnameof(g), "cluster", 7) ||
-         mapBool(agget(g, "cluster"), false);
+         mapbool(agget(g, "cluster"));
 }
 
-/* setAttr:
- * Sets object's name attribute to the given value.
+/** Sets object's name attribute to the given value.
  * Creates the attribute if not already set.
  */
 Agsym_t *setAttr(graph_t * g, void *obj, char *name, char *value,
@@ -794,13 +704,13 @@ Agsym_t *setAttr(graph_t * g, void *obj, char *name, char *value,
     if (ap == NULL) {
 	switch (agobjkind(obj)) {
 	case AGRAPH:
-	    ap = agattr(g, AGRAPH,name, "");
+	    ap = agattr_text(g, AGRAPH,name, "");
 	    break;
 	case AGNODE:
-	    ap = agattr(g,AGNODE, name, "");
+	    ap = agattr_text(g,AGNODE, name, "");
 	    break;
 	case AGEDGE:
-	    ap = agattr(g,AGEDGE, name, "");
+	    ap = agattr_text(g,AGEDGE, name, "");
 	    break;
 	}
     }
@@ -808,8 +718,7 @@ Agsym_t *setAttr(graph_t * g, void *obj, char *name, char *value,
     return ap;
 }
 
-/* clustNode:
- * Generate a special cluster node representing the end node
+/** Generate a special cluster node representing the end node
  * of an edge to the cluster cg. n is a node whose name is the same
  * as the cluster cg. clg is the subgraph of all of
  * the original nodes, which will be deleted later.
@@ -844,28 +753,22 @@ typedef struct {
     node_t *h;
 } item;
 
-static int cmpItem(Dt_t * d, void *p1[], void *p2[], Dtdisc_t * disc)
-{
-    (void)d;
-    (void)disc;
-
-    if (p1[0] < p2[0])
+static int cmpItem(void *pp1, void *pp2) {
+    const void **p1 = pp1;
+    const void **p2 = pp2;
+    if ((uintptr_t)p1[0] < (uintptr_t)p2[0])
 	return -1;
-    else if (p1[0] > p2[0])
+    if ((uintptr_t)p1[0] > (uintptr_t)p2[0])
 	return 1;
-    else if (p1[1] < p2[1])
+    if ((uintptr_t)p1[1] < (uintptr_t)p2[1])
 	return -1;
-    else if (p1[1] > p2[1])
+    if ((uintptr_t)p1[1] > (uintptr_t)p2[1])
 	return 1;
-    else
-	return 0;
+    return 0;
 }
 
-/* newItem:
- */
-static void *newItem(Dt_t * d, item * objp, Dtdisc_t * disc)
-{
-    (void)d;
+static void *newItem(void *p, Dtdisc_t *disc) {
+    item *objp = p;
     item *newp = gv_alloc(sizeof(item));
 
     (void)disc;
@@ -877,72 +780,51 @@ static void *newItem(Dt_t * d, item * objp, Dtdisc_t * disc)
     return newp;
 }
 
-/* freeItem:
- */
-static void freeItem(Dt_t * d, item * obj, Dtdisc_t * disc)
-{
-    (void)d;
-    (void)disc;
-    free(obj);
-}
-
 static Dtdisc_t mapDisc = {
     .key = offsetof(item, p),
     .size = sizeof(2 * sizeof(void *)),
     .link = offsetof(item, link),
-    .makef = (Dtmake_f)newItem,
-    .freef = (Dtfree_f)freeItem,
-    .comparf = (Dtcompar_f)cmpItem,
+    .makef = newItem,
+    .freef = free,
+    .comparf = cmpItem,
 };
 
-/* cloneEdge:
- * Make a copy of e in e's graph but using ct and ch as nodes
- */
+/// Make a copy of e in e's graph but using ct and ch as nodes
 static edge_t *cloneEdge(edge_t * e, node_t * ct, node_t * ch)
 {
     graph_t *g = agraphof(ct);
     edge_t *ce = agedge(g, ct, ch,NULL,1);
     agbindrec(ce, "Agedgeinfo_t", sizeof(Agedgeinfo_t), true);
     agcopyattr(e, ce);
-    ED_compound(ce) = TRUE;
+    ED_compound(ce) = true;
 
     return ce;
 }
 
-/* insertEdge:
- */
 static void insertEdge(Dt_t * map, void *t, void *h, edge_t * e)
 {
-    item dummy;
+    item dummy1 = {.p = {t, h}, .t = agtail(e), .h = aghead(e)};
+    dtinsert(map, &dummy1);
 
-    dummy.p[0] = t;
-    dummy.p[1] = h;
-    dummy.t = agtail(e);
-    dummy.h = aghead(e);
-    dtinsert(map, &dummy);
-
-    dummy.p[0] = h;
-    dummy.p[1] = t;
-    dummy.t = aghead(e);
-    dummy.h = agtail(e);
-    dtinsert(map, &dummy);
+    item dummy2 = {.p = {h, t}, .t = aghead(e), .h = agtail(e)};
+    dtinsert(map, &dummy2);
 }
 
-/* mapEdge:
- * Check if we already have cluster edge corresponding to t->h,
- * and return it.
- */
+/// Check if we already have cluster edge corresponding to t->h, and return it.
 static item *mapEdge(Dt_t * map, edge_t * e)
 {
-    void *key[2];
-
-    key[0] = agtail(e);
-    key[1] = aghead(e);
+    void *key[] = {agtail(e), aghead(e)};
     return dtmatch(map, &key);
 }
 
-/* checkCompound:
- * If endpoint names a cluster, mark for temporary deletion and create
+static graph_t *mapc(Dt_t *cmap, node_t *n) {
+  if (startswith(agnameof(n), "cluster")) {
+    return findCluster(cmap, agnameof(n));
+  }
+  return NULL;
+}
+
+/** If endpoint names a cluster, mark for temporary deletion and create
  * special node and insert into cluster. Then clone the edge. Real edge
  * will be deleted when we delete the original node.
  * Invariant: new edge has same sense as old. That is, given t->h with
@@ -957,13 +839,9 @@ static item *mapEdge(Dt_t * map, edge_t * e)
  *
  * Return 1 if cluster edge is created.
  */
-#define MAPC(n) (strncmp(agnameof(n),"cluster",7)?NULL:findCluster(cmap,agnameof(n)))
-
 static int
 checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
 {
-    graph_t *tg;
-    graph_t *hg;
     node_t *cn;
     node_t *cn1;
     node_t *t = agtail(e);
@@ -972,12 +850,12 @@ checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
     item *ip;
 
     if (IS_CLUST_NODE(h)) return 0;
-    tg = MAPC(t);
-    hg = MAPC(h);
+    graph_t *const tg = mapc(cmap, t);
+    graph_t *const hg = mapc(cmap, h);
     if (!tg && !hg)
 	return 0;
     if (tg == hg) {
-	agerr(AGWARN, "cluster cycle %s -- %s not supported\n", agnameof(t),
+	agwarningf("cluster cycle %s -- %s not supported\n", agnameof(t),
 	      agnameof(t));
 	return 0;
     }
@@ -990,12 +868,12 @@ checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
     if (hg) {
 	if (tg) {
 	    if (agcontains(hg, tg)) {
-		agerr(AGWARN, "tail cluster %s inside head cluster %s\n",
+		agwarningf("tail cluster %s inside head cluster %s\n",
 		      agnameof(tg), agnameof(hg));
 		return 0;
 	    }
 	    if (agcontains(tg, hg)) {
-		agerr(AGWARN, "head cluster %s inside tail cluster %s\n",
+		agwarningf("head cluster %s inside tail cluster %s\n",
 		      agnameof(hg),agnameof(tg));
 		return 0;
 	    }
@@ -1005,7 +883,7 @@ checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
 	    insertEdge(map, t, h, ce);
 	} else {
 	    if (agcontains(hg, t)) {
-		agerr(AGWARN, "tail node %s inside head cluster %s\n",
+		agwarningf("tail node %s inside head cluster %s\n",
 		      agnameof(t), agnameof(hg));
 		return 0;
 	    }
@@ -1015,7 +893,7 @@ checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
 	}
     } else {
 	if (agcontains(tg, h)) {
-	    agerr(AGWARN, "head node %s inside tail cluster %s\n", agnameof(h),
+	    agwarningf("head node %s inside tail cluster %s\n", agnameof(h),
 		  agnameof(tg));
 	    return 0;
 	}
@@ -1037,12 +915,10 @@ num_clust_edges(graph_t * g)
     cl_edge_t* cl_info = (cl_edge_t*)HAS_CLUST_EDGE(g);
     if (cl_info)
 	return cl_info->n_cluster_edges;
-    else
-	return 0;
+    return 0;
 }
 
-/* processClusterEdges:
- * Look for cluster edges. Replace cluster edge endpoints
+/** Look for cluster edges. Replace cluster edge endpoints
  * corresponding to a cluster with special cluster nodes.
  * Delete original nodes.
  * If cluster edges are found, a cl_edge_t record will be
@@ -1083,8 +959,7 @@ void processClusterEdges(graph_t * g)
     dtclose(cmap);
 }
 
-/* mapN:
- * Convert cluster nodes back to ordinary nodes
+/** Convert cluster nodes back to ordinary nodes
  * If n is already ordinary, return it.
  * Otherwise, we know node's name is "__i:xxx"
  * where i is some number and xxx is the nodes's original name.
@@ -1144,8 +1019,7 @@ static void undoCompound(edge_t * e, graph_t * clg)
     gv_cleanup_edge(e);
 }
 
-/* undoClusterEdges:
- * Replace cluster nodes with originals. Make sure original has
+/** Replace cluster nodes with originals. Make sure original has
  * no attributes. Replace original edges. Delete cluster nodes,
  * which will also delete cluster edges.
  */
@@ -1180,52 +1054,43 @@ void undoClusterEdges(graph_t * g)
     agclose(clg);
 }
 
-/* safe_dcl:
- * Find the attribute belonging to graph g for objects like obj
+/** Find the attribute belonging to graph g for objects like obj
  * with given name. If one does not exist, create it with the
  * default value defaultValue.
  */
 attrsym_t *safe_dcl(graph_t *g, int obj_kind, char *name, char *defaultValue) {
-    attrsym_t *a = agattr(g,obj_kind,name, NULL);
+    attrsym_t *a = agattr_text(g,obj_kind,name, NULL);
     if (!a)	/* attribute does not exist */
-        a = agattr(g, obj_kind, name, defaultValue);
+        a = agattr_text(g, obj_kind, name, defaultValue);
     return a;
 }
 
 static int comp_entities(const void *e1, const void *e2) {
-  return strcmp(((const struct entities_s *)e1)->name, ((const struct entities_s *)e2)->name);
+  const strview_t *key = e1;
+  const struct entities_s *candidate = e2;
+  return strview_cmp(*key, strview(candidate->name, '\0'));
 }
 
-#define MAXENTLEN 8
-
-/* scanEntity:
- * Scan non-numeric entity, convert to &#...; form and store in xbuf.
+/** Scan non-numeric entity, convert to &#...; form and store in xbuf.
  * t points to first char after '&'. Return after final semicolon.
  * If unknown, we return t and let libexpat flag the error.
  *     */
 char* scanEntity (char* t, agxbuf* xb)
 {
-    char*  endp = strchr (t, ';');
-    struct entities_s key, *res;
-    size_t len;
-    char   buf[MAXENTLEN+1];
+    const strview_t key = strview(t, ';');
+    struct entities_s *res;
 
     agxbputc(xb, '&');
-    if (!endp) return t;
-    if ((len = (size_t)(endp - t)) > MAXENTLEN || len < 2) return t;
-    strncpy (buf, t, len);
-    buf[len] = '\0';
-    key.name =  buf;
+    if (key.data[key.size] == '\0') return t;
+    if (key.size > ENTITY_NAME_LENGTH_MAX || key.size < 2) return t;
     res = bsearch(&key, entities, NR_OF_ENTITIES,
         sizeof(entities[0]), comp_entities);
     if (!res) return t;
     agxbprint(xb, "#%d;", res->value);
-    return endp + 1;
+    return t + key.size + 1;
 }
 
-
-/* htmlEntity:
- * Check for an HTML entity for a special character.
+/** Check for an HTML entity for a special character.
  * Assume *s points to first byte after '&'.
  * If successful, return the corresponding value and update s to
  * point after the terminating ';'.
@@ -1234,9 +1099,7 @@ char* scanEntity (char* t, agxbuf* xb)
 static int
 htmlEntity (char** s)
 {
-    char *p;
-    struct entities_s key, *res;
-    char entity_name_buf[ENTITY_NAME_LENGTH_MAX+1];
+    struct entities_s *res;
     unsigned char* str = *(unsigned char**)s;
     unsigned int byte;
     int i, n = 0;
@@ -1275,21 +1138,20 @@ htmlEntity (char** s)
 	}
     }
     else {
-	key.name = p = entity_name_buf;
+	strview_t key = {.data = (char *)str};
 	for (i = 0; i <  ENTITY_NAME_LENGTH_MAX; i++) {
 	    byte = *(str + i);
 	    if (byte == '\0') break;
 	    if (byte == ';') {
-		*p++ = '\0';
 		res = bsearch(&key, entities, NR_OF_ENTITIES,
-		    sizeof(entities[0]), *comp_entities);
+		    sizeof(entities[0]), comp_entities);
 		if (res) {
 		    n = res->value;
 		    str += i+1;
 		}
 		break;
 	    }
-	    *p++ = (char)byte;
+	    ++key.size;
 	}
     }
     *s = (char*)str;
@@ -1299,11 +1161,7 @@ htmlEntity (char** s)
 static unsigned char
 cvtAndAppend (unsigned char c, agxbuf* xb)
 {
-    char buf[2];
-
-    buf[0] = c;
-    buf[1] = '\0';
-
+    char buf[] = {c, '\0'};
     char *s = latin1ToUTF8(buf);
     char *p = s;
     size_t len = strlen(s);
@@ -1314,15 +1172,14 @@ cvtAndAppend (unsigned char c, agxbuf* xb)
     return c;
 }
 
-/* htmlEntityUTF8:
- * substitute html entities like: &#123; and: &amp; with the UTF8 equivalents
+/** substitute html entities like: &#123; and: &amp; with the UTF8 equivalents
  * check for invalid utf8. If found, treat a single byte as Latin-1, convert it to
  * utf8 and warn the user.
  */
 char* htmlEntityUTF8 (char* s, graph_t* g)
 {
     static graph_t* lastg;
-    static bool warned;
+    static atomic_flag warned;
     unsigned char c;
     unsigned int v;
 
@@ -1331,7 +1188,7 @@ char* htmlEntityUTF8 (char* s, graph_t* g)
 
     if (lastg != g) {
 	lastg = g;
-	warned = false;
+	atomic_flag_clear(&warned);
     }
 
     agxbuf xb = {0};
@@ -1353,9 +1210,8 @@ char* htmlEntityUTF8 (char* s, graph_t* g)
             uc = 3;
         else {
             uc = -1;
-            if (!warned) {
-                agerr(AGWARN, "UTF8 codes > 4 bytes are not currently supported (graph %s) - treated as Latin-1. Perhaps \"-Gcharset=latin1\" is needed?\n", agnameof(g));
-                warned = true;
+            if (!atomic_flag_test_and_set(&warned)) {
+                agwarningf("UTF8 codes > 4 bytes are not currently supported (graph %s) - treated as Latin-1. Perhaps \"-Gcharset=latin1\" is needed?\n", agnameof(g));
             }
             c = cvtAndAppend (c, &xb);
         }
@@ -1385,9 +1241,8 @@ char* htmlEntityUTF8 (char* s, graph_t* g)
                     c = *(unsigned char*)s++;
                 }
                 else {
-		            if (!warned) {
-		                agerr(AGWARN, "Invalid %d-byte UTF8 found in input of graph %s - treated as Latin-1. Perhaps \"-Gcharset=latin1\" is needed?\n", uc + 1, agnameof(g));
-		                warned = true;
+		            if (!atomic_flag_test_and_set(&warned)) {
+		                agwarningf("Invalid %d-byte UTF8 found in input of graph %s - treated as Latin-1. Perhaps \"-Gcharset=latin1\" is needed?\n", uc + 1, agnameof(g));
 		            }
 		            c = cvtAndAppend (c, &xb);
                     break;
@@ -1397,11 +1252,7 @@ char* htmlEntityUTF8 (char* s, graph_t* g)
     return agxbdisown(&xb);
 }
 
-/* latin1ToUTF8:
- * Converts string from Latin1 encoding to utf8
- * Also translates HTML entities.
- *
- */
+/// Converts string from Latin1 encoding to utf8. Also translates HTML entities.
 char* latin1ToUTF8 (char* s)
 {
     agxbuf xb = {0};
@@ -1430,8 +1281,7 @@ char* latin1ToUTF8 (char* s)
     return agxbdisown(&xb);
 }
 
-/* utf8ToLatin1:
- * Converts string from utf8 encoding to Latin1
+/** Converts string from utf8 encoding to Latin1
  * Note that it does not attempt to reproduce HTML entities.
  * We assume the input string comes from latin1ToUTF8.
  */
@@ -1468,9 +1318,7 @@ bool overlap_node(node_t *n, boxf b) {
 
 bool overlap_label(textlabel_t *lp, boxf b)
 {
-    pointf s;
-    s.x = lp->dimen.x / 2.;
-    s.y = lp->dimen.y / 2.;
+    const pointf s = {.x = lp->dimen.x / 2.0, .y = lp->dimen.y / 2.0};
     boxf bb = {.LL = sub_pointf(lp->pos, s), .UR = add_pointf(lp->pos, s)};
     return OVERLAP(b, bb);
 }
@@ -1484,7 +1332,7 @@ static bool overlap_arrow(pointf p, pointf u, double scale, boxf b)
 static bool overlap_bezier(bezier bz, boxf b) {
     assert(bz.size);
     pointf u = bz.list[0];
-    for (int i = 1; i < bz.size; i++) {
+    for (size_t i = 1; i < bz.size; i++) {
         pointf p = bz.list[i];
         if (lineToBox(p, u, b) != -1)
             return true;
@@ -1507,7 +1355,7 @@ bool overlap_edge(edge_t *e, boxf b)
 {
     splines *spl = ED_spl(e);
     if (spl && boxf_overlap(spl->bb, b))
-        for (int i = 0; i < spl->size; i++)
+        for (size_t i = 0; i < spl->size; i++)
             if (overlap_bezier(spl->list[i], b))
                 return true;
 
@@ -1518,9 +1366,7 @@ bool overlap_edge(edge_t *e, boxf b)
     return false;
 }
 
-/* edgeType:
- * Convert string to edge type.
- */
+/// Convert string to edge type.
 static int edgeType(const char *s, int defaultValue) {
     if (s == NULL || strcmp(s, "") == 0) {
         return defaultValue;
@@ -1554,12 +1400,11 @@ static int edgeType(const char *s, int defaultValue) {
 	return EDGETYPE_SPLINE;
     }
 
-    agerr(AGWARN, "Unknown \"splines\" value: \"%s\" - ignored\n", s);
+    agwarningf("Unknown \"splines\" value: \"%s\" - ignored\n", s);
     return defaultValue;
 }
 
-/* setEdgeType:
- * Sets graph's edge type based on the "splines" attribute.
+/** Sets graph's edge type based on the "splines" attribute.
  * If the attribute is not defined, use defaultValue.
  * If the attribute is "", use NONE.
  * If attribute value matches (case indepedent), use match.
@@ -1586,8 +1431,7 @@ void setEdgeType(graph_t *g, int defaultValue) {
     GD_flags(g) |= et;
 }
 
-/* get_gradient_points
- * Evaluates the extreme points of an ellipse or polygon
+/** Evaluates the extreme points of an ellipse or polygon
  * Determines the point at the center of the extreme points
  * If isRadial is true,sets the inner radius to half the distance to the min point;
  * else uses the angle parameter to identify two points on a line that defines the
@@ -1595,7 +1439,7 @@ void setEdgeType(graph_t *g, int defaultValue) {
  * By default, this assumes a left-hand coordinate system (for svg); if RHS = 2 flag
  * is set, use standard coordinate system.
  */
-void get_gradient_points(pointf *A, pointf *G, int n, double angle, int flags) {
+void get_gradient_points(pointf *A, pointf *G, size_t n, double angle, int flags) {
     pointf min,max,center;
     int isRadial = flags & 1;
     int isRHS = flags & 2;
@@ -1611,7 +1455,7 @@ void get_gradient_points(pointf *A, pointf *G, int n, double angle, int flags) {
     else {
       min.x = max.x = A[0].x;
       min.y = max.y = A[0].y;
-      for (int i = 0; i < n; i++) {
+      for (size_t i = 0; i < n; i++) {
             min.x = MIN(A[i].x, min.x);
             min.y = MIN(A[i].y, min.y);
             max.x = MAX(A[i].x, max.x);
@@ -1654,7 +1498,7 @@ void get_gradient_points(pointf *A, pointf *G, int n, double angle, int flags) {
 
 void gv_free_splines(edge_t *e) {
     if (ED_spl(e)) {
-        for (int i = 0; i < ED_spl(e)->size; i++)
+        for (size_t i = 0; i < ED_spl(e)->size; i++)
             free(ED_spl(e)->list[i].list);
         free(ED_spl(e)->list);
         free(ED_spl(e));
@@ -1713,18 +1557,11 @@ typedef struct {
     Agraph_t* clp;
 } clust_t;
 
-static void free_clust (Dt_t* dt, clust_t* clp, Dtdisc_t* disc)
-{
-    (void)dt;
-    (void)disc;
-    free(clp);
-}
-
 static Dtdisc_t strDisc = {
     .key = offsetof(clust_t, name),
     .size = -1,
     .link = offsetof(clust_t, link),
-    .freef = (Dtfree_f)free_clust,
+    .freef = free,
 };
 
 static void fillMap (Agraph_t* g, Dt_t* map)
@@ -1733,7 +1570,7 @@ static void fillMap (Agraph_t* g, Dt_t* map)
         Agraph_t *cl = GD_clust(g)[c];
         char *s = agnameof(cl);
         if (dtmatch(map, s)) {
-            agerr(AGWARN, "Two clusters named %s - the second will be ignored\n", s);
+            agwarningf("Two clusters named %s - the second will be ignored\n", s);
         } else {
             clust_t *ip = gv_alloc(sizeof(clust_t));
             ip->name = s;
@@ -1744,8 +1581,7 @@ static void fillMap (Agraph_t* g, Dt_t* map)
     }
 }
 
-/* mkClustMap:
- * Generates a dictionary mapping cluster names to corresponding cluster.
+/** Generates a dictionary mapping cluster names to corresponding cluster.
  * Used with cgraph as the latter does not support a flat namespace of clusters.
  * Assumes G has already built a cluster tree using GD_n_cluster and GD_clust.
  */
@@ -1764,11 +1600,17 @@ findCluster (Dt_t* map, char* name)
     clust_t* clp = dtmatch (map, name);
     if (clp)
 	return clp->clp;
-    else
-	return NULL;
+    return NULL;
 }
 
 /**
  * @dir lib/common
  * @brief common code for layout engines
+ * @ingroup engines
+ *
+ * @ref common_utils
+ *
+ * @defgroup common_utils utilities
+ * @brief low level utilities for layout engines and rendering
+ * @ingroup engines
  */
