@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -11,13 +11,15 @@
 #include <assert.h>
 #include <dotgen/dot.h>
 #include <stdbool.h>
+#include <stddef.h>
+#include <util/alloc.h>
 
 static node_t*
 map_interclust_node(node_t * n)
 {
     node_t *rv;
 
-    if ((ND_clust(n) == NULL) || (  GD_expanded(ND_clust(n))) )
+    if (ND_clust(n) == NULL || GD_expanded(ND_clust(n)))
 	rv = n;
     else
 	rv = GD_rankleader(ND_clust(n))[ND_rank(n)];
@@ -78,16 +80,15 @@ map_path(node_t * from, node_t * to, edge_t * orig, edge_t * ve, int type)
 
     assert(ND_rank(from) < ND_rank(to));
 
-    if ((agtail(ve) == from) && (aghead(ve) == to))
+    if (agtail(ve) == from && aghead(ve) == to)
 	return;
 
     if (ED_count(ve) > 1) {
 	ED_to_virt(orig) = NULL;
 	if (ND_rank(to) - ND_rank(from) == 1) {
-	    if ((e = find_fast_edge(from, to)) && (ports_eq(orig, e))) {
+	    if ((e = find_fast_edge(from, to)) && ports_eq(orig, e)) {
 		merge_oneway(orig, e);
-		if ((ND_node_type(from) == NORMAL)
-		    && (ND_node_type(to) == NORMAL))
+		if (ND_node_type(from) == NORMAL && ND_node_type(to) == NORMAL)
 		    other_edge(orig);
 		return;
 	    }
@@ -106,13 +107,12 @@ map_path(node_t * from, node_t * to, edge_t * orig, edge_t * ve, int type)
 	}
     } else {
 	if (ND_rank(to) - ND_rank(from) == 1) {
-	    if ((ve = find_fast_edge(from, to)) && (ports_eq(orig, ve))) {
+	    if ((ve = find_fast_edge(from, to)) && ports_eq(orig, ve)) {
 		/*ED_to_orig(ve) = orig; */
 		ED_to_virt(orig) = ve;
 		ED_edge_type(ve) = type;
 		ED_count(ve)++;
-		if ((ND_node_type(from) == NORMAL)
-		    && (ND_node_type(to) == NORMAL))
+		if (ND_node_type(from) == NORMAL && ND_node_type(to) == NORMAL)
 		    other_edge(orig);
 	    } else {
 		ED_to_virt(orig) = NULL;
@@ -145,7 +145,7 @@ static void make_interclust_chain(node_t * from, node_t * to, edge_t * orig) {
 
     u = map_interclust_node(from);
     v = map_interclust_node(to);
-    if ((u == from) && (v == to))
+    if (u == from && v == to)
 	newtype = VIRTUAL;
     else
 	newtype = CLUSTER_EDGE;
@@ -183,7 +183,7 @@ static void interclexp(graph_t * subg)
 		if (ED_to_virt(prev) == NULL)
 		    continue;	/* internal edge */
 		ED_to_virt(e) = NULL;
-		merge_chain(subg, e, ED_to_virt(prev), FALSE);
+		merge_chain(subg, e, ED_to_virt(prev), false);
 		safe_other_edge(e);
 		continue;
 	    }
@@ -246,7 +246,6 @@ merge_ranks(graph_t * subg)
 		v->root = agroot(root);
 	    delete_fast_node(subg, v);
 	    fast_node(root, v);
-	    GD_n_nodes(root)++;
 	}
 	GD_rank(subg)[r].v = GD_rank(root)[r].v + ipos;
 	GD_rank(root)[r].valid = false;
@@ -286,19 +285,24 @@ remove_rankleaders(graph_t * g)
 }
 
 /* delete virtual nodes of a cluster, and install real nodes or sub-clusters */
-void expand_cluster(graph_t * subg)
-{
+int expand_cluster(graph_t *subg) {
     /* build internal structure of the cluster */
     class2(subg);
     GD_comp(subg).size = 1;
     GD_comp(subg).list[0] = GD_nlist(subg);
     allocate_ranks(subg);
-    build_ranks(subg, 0);
+    ints_t scratch = {0};
+    const int rc = build_ranks(subg, 0, &scratch);
+    ints_free(&scratch);
+    if (rc != 0) {
+        return rc;
+    }
     merge_ranks(subg);
 
     /* build external structure of the cluster */
     interclexp(subg);
     remove_rankleaders(subg);
+    return 0;
 }
 
 /* this function marks every node in <g> with its top-level cluster under <g> */
@@ -321,7 +325,7 @@ void mark_clusters(graph_t * g)
 	for (n = agfstnode(clust); n; n = nn) {
 		nn = agnxtnode(clust,n);
 	    if (ND_ranktype(n) != NORMAL) {
-		agerr(AGWARN,
+		agwarningf(
 		      "%s was already in a rankset, deleted from cluster %s\n",
 		      agnameof(n), agnameof(g));
 		agdelete(clust,n);
@@ -353,7 +357,7 @@ void build_skeleton(graph_t * g, graph_t * subg)
     edge_t *e;
 
     prev = NULL;
-    GD_rankleader(subg) = N_NEW(GD_maxrank(subg) + 2, node_t *);
+    GD_rankleader(subg) = gv_calloc(GD_maxrank(subg) + 2, sizeof(node_t*));
     for (r = GD_minrank(subg); r <= GD_maxrank(subg); r++) {
 	v = GD_rankleader(subg)[r] = virtual_node(g);
 	ND_rank(v) = r;
@@ -383,19 +387,23 @@ void build_skeleton(graph_t * g, graph_t * subg)
     }
 }
 
-void install_cluster(graph_t * g, node_t * n, int pass, nodequeue * q)
-{
+int install_cluster(graph_t *g, node_t *n, int pass, node_queue_t *q) {
     int r;
     graph_t *clust;
 
     clust = ND_clust(n);
     if (GD_installed(clust) != pass + 1) {
-	for (r = GD_minrank(clust); r <= GD_maxrank(clust); r++)
-	    install_in_rank(g, GD_rankleader(clust)[r]);
+	for (r = GD_minrank(clust); r <= GD_maxrank(clust); r++) {
+	    const int rc = install_in_rank(g, GD_rankleader(clust)[r]);
+	    if (rc != 0) {
+	        return rc;
+	    }
+	}
 	for (r = GD_minrank(clust); r <= GD_maxrank(clust); r++)
 	    enqueue_neighbors(q, GD_rankleader(clust)[r], pass);
 	GD_installed(clust) = pass + 1;
     }
+    return 0;
 }
 
 static void mark_lowcluster_basic(Agraph_t * g);
