@@ -13,16 +13,18 @@
  */
 
 #include <assert.h>
-#include <cgraph/list.h>
 #include <common/boxes.h>
 #include <dotgen/dot.h>
 #include <math.h>
+#include <stdatomic.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <util/agxbuf.h>
 #include <util/alloc.h>
+#include <util/gv_math.h>
+#include <util/list.h>
 
 #ifdef ORTHO
 #include <ortho/ortho.h>
@@ -76,8 +78,8 @@ static bool cl_vninside(Agraph_t *, Agnode_t *);
 static void completeregularpath(path *, Agedge_t *, Agedge_t *, pathend_t *,
                                 pathend_t *, const boxes_t *);
 static int edgecmp(const void *, const void *);
-static void make_flat_edge(graph_t *, spline_info_t *, path *, Agedge_t **,
-                           unsigned, unsigned, int);
+static int make_flat_edge(graph_t *, spline_info_t *, path *, Agedge_t **,
+                          unsigned, unsigned, int);
 static void make_regular_edge(graph_t *g, spline_info_t *, path *, Agedge_t **,
                               unsigned, unsigned, int);
 static boxf makeregularend(boxf, int, double);
@@ -146,21 +148,11 @@ int portcmp(port p0, port p1) {
 static void swap_bezier(bezier *b) {
   const size_t sz = b->size;
   for (size_t i = 0; i < sz / 2; ++i) { // reverse list of points
-    pointf tmp = b->list[i];
-    b->list[i] = b->list[sz - 1 - i];
-    b->list[sz - 1 - i] = tmp;
+    SWAP(&b->list[i], &b->list[sz - 1 - i]);
   }
 
-  {
-    uint32_t tmp = b->sflag;
-    b->sflag = b->eflag;
-    b->eflag = tmp;
-  }
-  {
-    pointf tmp = b->sp;
-    b->sp = b->ep;
-    b->ep = tmp;
-  }
+  SWAP(&b->sflag, &b->eflag);
+  SWAP(&b->sp, &b->ep);
 }
 
 static void swap_spline(splines *s) {
@@ -168,9 +160,7 @@ static void swap_spline(splines *s) {
 
   // reverse list
   for (size_t i = 0; i < sz / 2; ++i) {
-    bezier tmp = s->list[i];
-    s->list[i] = s->list[sz - 1 - i];
-    s->list[sz - 1 - i] = tmp;
+    SWAP(&s->list[i], &s->list[sz - 1 - i]);
   }
 
   // swap beziers
@@ -179,8 +169,7 @@ static void swap_spline(splines *s) {
   }
 }
 
-/* edge_normalize:
- * Some back edges are reversed during layout and the reversed edge
+/* Some back edges are reversed during layout and the reversed edge
  * is used to compute the spline. We would like to guarantee that
  * the order of control points always goes from tail to head, so
  * we reverse them if necessary.
@@ -197,8 +186,7 @@ static void edge_normalize(graph_t *g) {
   }
 }
 
-/* resetRW:
- * In position, each node has its rw stored in mval and,
+/* In position, each node has its rw stored in mval and,
  * if a node is part of a loop, rw may be increased to
  * reflect the loops and associated labels. We restore
  * the original value here.
@@ -208,15 +196,12 @@ static void resetRW(graph_t *g) {
 
   for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
     if (ND_other(n).list) {
-      double tmp = ND_rw(n);
-      ND_rw(n) = ND_mval(n);
-      ND_mval(n) = tmp;
+      SWAP(&ND_rw(n), &ND_mval(n));
     }
   }
 }
 
-/* setEdgeLabelPos:
- * Set edge label position information for regular and non-adjacent flat edges.
+/* Set edge label position information for regular and non-adjacent flat edges.
  * Dot has allocated space and position for these labels. This info will be
  * used when routing orthogonal edges.
  */
@@ -247,8 +232,10 @@ static void setEdgeLabelPos(graph_t *g) {
  * recursive call in make_flat_edge without normalization occurring,
  * so that the edge will only be normalized once in the top level call
  * of dot_splines.
+ *
+ * @return 0 on success
  */
-static void dot_splines_(graph_t *g, int normalize) {
+static int dot_splines_(graph_t *g, int normalize) {
   int i, j, k, n_nodes;
   node_t *n;
   Agedgeinfo_t fwdedgeai, fwdedgebi;
@@ -260,7 +247,7 @@ static void dot_splines_(graph_t *g, int normalize) {
   fwdedgeb.out.base.data = (Agrec_t *)&fwdedgebi;
 
   if (et == EDGETYPE_NONE)
-    return;
+    return 0;
   if (et == EDGETYPE_CURVED) {
     resetRW(g);
     if (GD_has_labels(g->root) & EDGE_LABEL) {
@@ -284,7 +271,7 @@ static void dot_splines_(graph_t *g, int normalize) {
 
   mark_lowclusters(g);
   if (routesplinesinit())
-    return;
+    return 0;
   spline_info_t sd = {.Splinesep = GD_nodesep(g) / 4,
                       .Multisep = GD_nodesep(g)};
   edges = gv_calloc(CHUNK, sizeof(edge_t *));
@@ -295,9 +282,9 @@ static void dot_splines_(graph_t *g, int normalize) {
   for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
     n_nodes += GD_rank(g)[i].n;
     if ((n = GD_rank(g)[i].v[0]))
-      sd.LeftBound = MIN(sd.LeftBound, (ND_coord(n).x - ND_lw(n)));
+      sd.LeftBound = MIN(sd.LeftBound, ND_coord(n).x - ND_lw(n));
     if (GD_rank(g)[i].n && (n = GD_rank(g)[i].v[GD_rank(g)[i].n - 1]))
-      sd.RightBound = MAX(sd.RightBound, (ND_coord(n).x + ND_rw(n)));
+      sd.RightBound = MAX(sd.RightBound, ND_coord(n).x + ND_rw(n));
     sd.LeftBound -= MINW;
     sd.RightBound += MINW;
 
@@ -336,9 +323,7 @@ static void dot_splines_(graph_t *g, int normalize) {
          * the original value here.
          */
         if (ND_node_type(n) == NORMAL) {
-          double tmp = ND_rw(n);
-          ND_rw(n) = ND_mval(n);
-          ND_mval(n) = tmp;
+          SWAP(&ND_rw(n), &ND_mval(n));
         }
         for (k = 0; (e = ND_other(n).list[k]); k++) {
           setflags(e, 0, 0, AUXGRAPH);
@@ -440,7 +425,10 @@ static void dot_splines_(graph_t *g, int normalize) {
           updateBB(g, ED_label(e));
       }
     } else if (ND_rank(agtail(e0)) == ND_rank(aghead(e0))) {
-      make_flat_edge(g, &sd, &P, edges, ind, cnt, et);
+      const int rc = make_flat_edge(g, &sd, &P, edges, ind, cnt, et);
+      if (rc != 0) {
+        return rc;
+      }
     } else
       make_regular_edge(g, &sd, &P, edges, ind, cnt, et);
   }
@@ -495,15 +483,16 @@ finish:
   free(P.boxes);
   State = GVSPLINES;
   EdgeLabelsDone = 1;
+  return 0;
 }
 
-/* dot_splines:
- * If the splines attribute is defined but equal to "", skip edge routing.
+/* If the splines attribute is defined but equal to "", skip edge routing.
+ *
+ * @return 0 on success
  */
-void dot_splines(graph_t *g) { dot_splines_(g, 1); }
+int dot_splines(graph_t *g) { return dot_splines_(g, 1); }
 
-/* place_vnlabel:
- * assign position of an edge label from its virtual node
+/* assign position of an edge label from its virtual node
  * This is for regular edges only.
  */
 static void place_vnlabel(node_t *n) {
@@ -546,11 +535,10 @@ static void setflags(edge_t *e, int hint1, int hint2, int f3) {
     else /* f1 == SELF*EDGE */
       f2 = FWDEDGE;
   }
-  ED_tree_index(e) = (f1 | f2 | f3);
+  ED_tree_index(e) = f1 | f2 | f3;
 }
 
-/* edgecmp:
- * lexicographically order edges by
+/* lexicographically order edges by
  *  - edge type
  *  - |rank difference of nodes|
  *  - |x difference of nodes|
@@ -576,8 +564,7 @@ static int edgecmp(const void *x, const void *y) {
   Agedgeinfo_t fwdedgeai, fwdedgebi;
   Agedgepair_t fwdedgea, fwdedgeb;
   edge_t *e0, *e1, *ea, *eb, *le0, *le1;
-  int et0, et1, v0, v1, rv;
-  double t0, t1;
+  int et0, et1, rv;
 
   fwdedgea.out.base.data = (Agrec_t *)&fwdedgeai;
   fwdedgeb.out.base.data = (Agrec_t *)&fwdedgebi;
@@ -595,27 +582,30 @@ static int edgecmp(const void *x, const void *y) {
   le0 = getmainedge(e0);
   le1 = getmainedge(e1);
 
-  t0 = ND_rank(agtail(le0)) - ND_rank(aghead(le0));
-  t1 = ND_rank(agtail(le1)) - ND_rank(aghead(le1));
-  v0 = abs((int)t0); /* ugly, but explicit as to how we avoid equality tests on
-                        fp numbers */
-  v1 = abs((int)t1);
-  if (v0 < v1) {
-    return -1;
-  }
-  if (v0 > v1) {
-    return 1;
+  {
+    const int rank_diff0 = ND_rank(agtail(le0)) - ND_rank(aghead(le0));
+    const int rank_diff1 = ND_rank(agtail(le1)) - ND_rank(aghead(le1));
+    const int v0 = abs(rank_diff0);
+    const int v1 = abs(rank_diff1);
+    if (v0 < v1) {
+      return -1;
+    }
+    if (v0 > v1) {
+      return 1;
+    }
   }
 
-  t0 = ND_coord(agtail(le0)).x - ND_coord(aghead(le0)).x;
-  t1 = ND_coord(agtail(le1)).x - ND_coord(aghead(le1)).x;
-  v0 = abs((int)t0);
-  v1 = abs((int)t1);
-  if (v0 < v1) {
-    return -1;
-  }
-  if (v0 > v1) {
-    return 1;
+  {
+    const double t0 = ND_coord(agtail(le0)).x - ND_coord(aghead(le0)).x;
+    const double t1 = ND_coord(agtail(le1)).x - ND_coord(aghead(le1)).x;
+    const double v0 = fabs(t0);
+    const double v1 = fabs(t1);
+    if (v0 < v1) {
+      return -1;
+    }
+    if (v0 > v1) {
+      return 1;
+    }
   }
 
   /* This provides a cheap test for edges having the same set of endpoints.  */
@@ -757,12 +747,12 @@ static void setState(graph_t *auxg, attr_state_t *attr_state) {
   attr_state->G_ordering = G_ordering;
 
   E_constr = NULL;
-  E_dir = agattr(auxg, AGEDGE, "dir", NULL);
-  E_samehead = agattr(auxg, AGEDGE, "samehead", NULL);
-  E_sametail = agattr(auxg, AGEDGE, "sametail", NULL);
-  E_weight = agattr(auxg, AGEDGE, "weight", NULL);
+  E_dir = agattr_text(auxg, AGEDGE, "dir", NULL);
+  E_samehead = agattr_text(auxg, AGEDGE, "samehead", NULL);
+  E_sametail = agattr_text(auxg, AGEDGE, "sametail", NULL);
+  E_weight = agattr_text(auxg, AGEDGE, "weight", NULL);
   if (!E_weight)
-    E_weight = agattr(auxg, AGEDGE, "weight", "");
+    E_weight = agattr_text(auxg, AGEDGE, "weight", "");
   E_minlen = NULL;
   E_fontcolor = NULL;
   E_fontname = agfindedgeattr(auxg, "fontname");
@@ -799,8 +789,7 @@ static void setState(graph_t *auxg, attr_state_t *attr_state) {
   G_ordering = agfindgraphattr(auxg, "ordering");
 }
 
-/* cloneGraph:
- * Create clone graph. It stores the global Agsyms, to be
+/* Create clone graph. It stores the global Agsyms, to be
  * restored in cleanupCloneGraph. The graph uses the main
  * graph's settings for certain geometry parameters, and
  * declares all node and edge attributes used in the original
@@ -814,7 +803,7 @@ static graph_t *cloneGraph(graph_t *g, attr_state_t *attr_state) {
   else
     auxg = agopen("auxg", Agundirected, NULL);
   agbindrec(auxg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
-  agattr(auxg, AGRAPH, "rank", "");
+  agattr_text(auxg, AGRAPH, "rank", "");
   GD_drawing(auxg) = gv_alloc(sizeof(layout_t));
   GD_drawing(auxg)->quantum = GD_drawing(g)->quantum;
   GD_drawing(auxg)->dpi = GD_drawing(g)->dpi;
@@ -829,18 +818,24 @@ static graph_t *cloneGraph(graph_t *g, attr_state_t *attr_state) {
 
   // copy node attrs to auxg
   sym = agnxtattr(agroot(g), AGNODE, NULL); // get the first attr.
-  for (; sym; sym = agnxtattr(agroot(g), AGNODE, sym))
-    agattr(auxg, AGNODE, sym->name, sym->defval);
+  for (; sym; sym = agnxtattr(agroot(g), AGNODE, sym)) {
+    const bool is_html = aghtmlstr(sym->defval);
+    is_html ? agattr_html(auxg, AGNODE, sym->name, sym->defval)
+            : agattr_text(auxg, AGNODE, sym->name, sym->defval);
+  }
 
   // copy edge attributes
   sym = agnxtattr(agroot(g), AGEDGE, NULL); // get the first attr.
-  for (; sym; sym = agnxtattr(agroot(g), AGEDGE, sym))
-    agattr(auxg, AGEDGE, sym->name, sym->defval);
+  for (; sym; sym = agnxtattr(agroot(g), AGEDGE, sym)) {
+    const bool is_html = aghtmlstr(sym->defval);
+    is_html ? agattr_html(auxg, AGEDGE, sym->name, sym->defval)
+            : agattr_text(auxg, AGEDGE, sym->name, sym->defval);
+  }
 
-  if (!agattr(auxg, AGEDGE, "headport", NULL))
-    agattr(auxg, AGEDGE, "headport", "");
-  if (!agattr(auxg, AGEDGE, "tailport", NULL))
-    agattr(auxg, AGEDGE, "tailport", "");
+  if (!agattr_text(auxg, AGEDGE, "headport", NULL))
+    agattr_text(auxg, AGEDGE, "headport", "");
+  if (!agattr_text(auxg, AGEDGE, "tailport", NULL))
+    agattr_text(auxg, AGEDGE, "tailport", "");
 
   setState(auxg, attr_state);
 
@@ -894,8 +889,7 @@ static void cleanupCloneGraph(graph_t *g, attr_state_t *attr_state) {
   agclose(g);
 }
 
-/* cloneNode:
- * If original graph has rankdir=LR or RL, records change shape,
+/* If original graph has rankdir=LR or RL, records change shape,
  * so we wrap a record node's label in "{...}" to prevent this.
  */
 static node_t *cloneNode(graph_t *g, node_t *orign) {
@@ -920,9 +914,7 @@ static edge_t *cloneEdge(graph_t *g, node_t *tn, node_t *hn, edge_t *orig) {
   return e;
 }
 
-/* transformf:
- * Rotate, if necessary, then translate points.
- */
+/// rotate, if necessary, then translate points
 static pointf transformf(pointf p, pointf del, int flip) {
   if (flip) {
     double i = p.x;
@@ -932,8 +924,7 @@ static pointf transformf(pointf p, pointf del, int flip) {
   return add_pointf(p, del);
 }
 
-/* edgelblcmpfn:
- * lexicographically order edges by
+/* lexicographically order edges by
  *  - has label
  *  - label is wider
  *  - label is higher
@@ -980,8 +971,7 @@ static int edgelblcmpfn(const void *x, const void *y) {
 
 #define LBL_SPACE 6 /* space between labels, in points */
 
-/* makeSimpleFlatLabels:
- * This handles the second simplest case for flat edges between
+/* This handles the second simplest case for flat edges between
  * two adjacent nodes. We still invoke a dot on a rotated problem
  * to handle edges with ports. This usually works, but fails for
  * records because of their weird nature.
@@ -1181,17 +1171,18 @@ static void makeSimpleFlat(node_t *tn, node_t *hn, edge_t **edges, unsigned ind,
   }
 }
 
-/* make_flat_adj_edges:
- * In the simple case, with no labels or ports, this creates a simple
+/* In the simple case, with no labels or ports, this creates a simple
  * spindle of splines.
  * If there are only labels, cobble something together.
  * Otherwise, we run dot recursively on the 2 nodes and the edges,
  * essentially using rankdir=LR, to get the needed spline info.
  * This is probably to cute and fragile, and should be rewritten in a
  * more straightforward and laborious fashion.
+ *
+ * @return 0 on success
  */
-static void make_flat_adj_edges(graph_t *g, edge_t **edges, unsigned ind,
-                                unsigned cnt, edge_t *e0, int et) {
+static int make_flat_adj_edges(graph_t *g, edge_t **edges, unsigned ind,
+                               unsigned cnt, edge_t *e0, int et) {
   node_t *n;
   node_t *tn, *hn;
   edge_t *e;
@@ -1202,18 +1193,17 @@ static void make_flat_adj_edges(graph_t *g, edge_t **edges, unsigned ind,
   double midx, midy, leftx, rightx;
   pointf del;
   edge_t *hvye = NULL;
-  static bool warned = false;
+  static atomic_flag warned;
 
   tn = agtail(e0), hn = aghead(e0);
   if (shapeOf(tn) == SH_RECORD || shapeOf(hn) == SH_RECORD) {
-    if (!warned) {
-      warned = true;
+    if (!atomic_flag_test_and_set(&warned)) {
       agwarningf("flat edge between adjacent nodes one of which has a record "
                  "shape - replace records with HTML-like labels\n");
       agerr(AGPREV, "  Edge %s %s %s\n", agnameof(tn),
             agisdirected(g) ? "->" : "--", agnameof(hn));
     }
-    return;
+    return 0;
   }
   unsigned labels = 0;
   bool ports = false;
@@ -1234,7 +1224,7 @@ static void make_flat_adj_edges(graph_t *g, edge_t **edges, unsigned ind,
     else {
       makeSimpleFlatLabels(tn, hn, edges, ind, cnt, et, labels);
     }
-    return;
+    return 0;
   }
 
   attr_state_t attrs = {0};
@@ -1245,9 +1235,7 @@ static void make_flat_adj_edges(graph_t *g, edge_t **edges, unsigned ind,
   rightx = ND_coord(hn).x;
   leftx = ND_coord(tn).x;
   if (GD_flip(g)) {
-    node_t *tmp = tn;
-    tn = hn;
-    hn = tmp;
+    SWAP(&tn, &hn);
   }
   auxt = cloneNode(subg, tn);
   auxh = cloneNode(auxg, hn);
@@ -1275,7 +1263,10 @@ static void make_flat_adj_edges(graph_t *g, edge_t **edges, unsigned ind,
   dot_init_node_edge(auxg);
 
   dot_rank(auxg);
-  dot_mincross(auxg);
+  const int r = dot_mincross(auxg);
+  if (r != 0) {
+    return r;
+  }
   dot_position(auxg);
 
   /* reposition */
@@ -1292,7 +1283,10 @@ static void make_flat_adj_edges(graph_t *g, edge_t **edges, unsigned ind,
       ND_coord(n).y = midx;
   }
   dot_sameports(auxg);
-  dot_splines_(auxg, 0);
+  const int rc = dot_splines_(auxg, 0);
+  if (rc != 0) {
+    return rc;
+  }
   dotneato_postprocess(auxg);
 
   /* copy splines */
@@ -1340,6 +1334,7 @@ static void make_flat_adj_edges(graph_t *g, edge_t **edges, unsigned ind,
   }
 
   cleanupCloneGraph(auxg, &attrs);
+  return 0;
 }
 
 static void makeFlatEnd(graph_t *g, spline_info_t *sp, path *P, node_t *n,
@@ -1554,17 +1549,18 @@ static void make_flat_bottom_edges(graph_t *g, spline_info_t *sp, path *P,
   }
 }
 
-/* make_flat_edge:
- * Construct flat edges edges[ind...ind+cnt-1]
+/* Construct flat edges edges[ind...ind+cnt-1]
  * There are 4 main cases:
  *  - all edges between a and b where a and b are adjacent
  *  - one labeled edge
  *  - all non-labeled edges with identical ports between non-adjacent a and b
  *     = connecting bottom to bottom/left/right - route along bottom
  *     = the rest - route along top
+ *
+ * @return 0 on success
  */
-static void make_flat_edge(graph_t *g, spline_info_t *sp, path *P,
-                           edge_t **edges, unsigned ind, unsigned cnt, int et) {
+static int make_flat_edge(graph_t *g, spline_info_t *sp, path *P,
+                          edge_t **edges, unsigned ind, unsigned cnt, int et) {
   node_t *tn, *hn;
   Agedgeinfo_t fwdedgei;
   Agedgepair_t fwdedge;
@@ -1593,24 +1589,23 @@ static void make_flat_edge(graph_t *g, spline_info_t *sp, path *P,
    * so check them all.
    */
   if (isAdjacent) {
-    make_flat_adj_edges(g, edges, ind, cnt, e, et);
-    return;
+    return make_flat_adj_edges(g, edges, ind, cnt, e, et);
   }
   if (ED_label(e)) { /* edges with labels aren't multi-edges */
     make_flat_labeled_edge(g, sp, P, e, et);
-    return;
+    return 0;
   }
 
   if (et == EDGETYPE_LINE) {
     makeSimpleFlat(agtail(e), aghead(e), edges, ind, cnt, et);
-    return;
+    return 0;
   }
 
   tside = ED_tail_port(e).side;
   hside = ED_head_port(e).side;
   if ((tside == BOTTOM && hside != TOP) || (hside == BOTTOM && tside != TOP)) {
     make_flat_bottom_edges(g, sp, P, edges, ind, cnt, e, et == EDGETYPE_SPLINE);
-    return;
+    return 0;
   }
 
   tn = agtail(e);
@@ -1674,12 +1669,13 @@ static void make_flat_edge(graph_t *g, spline_info_t *sp, path *P,
       ps = routepolylines(P, &pn);
     if (pn == 0) {
       free(ps);
-      return;
+      return 0;
     }
     clip_and_install(e, aghead(e), ps, pn, &sinfo);
     free(ps);
     P->nbox = 0;
   }
+  return 0;
 }
 
 /// Return true if p3 is to left of ray p1->p2
@@ -1687,8 +1683,7 @@ static bool leftOf(pointf p1, pointf p2, pointf p3) {
   return (p1.y - p2.y) * (p3.x - p2.x) - (p3.y - p2.y) * (p1.x - p2.x) > 0;
 }
 
-/* makeLineEdge:
- * Create an edge as line segment. We guarantee that the points
+/* Create an edge as line segment. We guarantee that the points
  * are always drawn downwards. This means that for flipped edges,
  * we draw from the head to the tail. The routine returns the
  * end node of the edge in *hp. The points are stored in the
@@ -2016,8 +2011,7 @@ static void completeregularpath(path *P, edge_t *first, edge_t *last,
   adjustregularpath(P, fb, lb);
 }
 
-/* makeregularend:
- * Add box to fill between node and interrank space. Needed because
+/* Add box to fill between node and interrank space. Needed because
  * nodes in a given rank can differ in height.
  * for now, regular edges always go from top to bottom
  */
@@ -2029,8 +2023,7 @@ static boxf makeregularend(boxf b, int side, double y) {
   return (boxf){{b.LL.x, b.UR.y}, {b.UR.x, y}};
 }
 
-/* adjustregularpath:
- * make sure the path is wide enough.
+/* make sure the path is wide enough.
  * the % 2 was so that in rank boxes would only be grown if
  * they were == 0 while inter-rank boxes could be stretched to a min
  * width.
@@ -2232,8 +2225,7 @@ static Agraph_t *cl_bound(graph_t *g, node_t *n, node_t *adj) {
   return rv;
 }
 
-/* maximal_bbox:
- * Return an initial bounding box to be used for building the
+/* Return an initial bounding box to be used for building the
  * beginning or ending of the path of boxes.
  * Height reflects height of tallest node on rank.
  * The extra space provided by FUDGE allows begin/endpath to create a box
