@@ -14,7 +14,6 @@
  */
 
 #include <assert.h>
-#include <cgraph/gv_ctype.h>
 #include <neatogen/neato.h>
 #include <common/utils.h>
 #include <float.h>
@@ -36,6 +35,8 @@
 #include <stddef.h>
 #include <util/agxbuf.h>
 #include <util/alloc.h>
+#include <util/debug.h>
+#include <util/gv_ctype.h>
 #include <util/startswith.h>
 #include <util/strcasecmp.h>
 
@@ -467,10 +468,8 @@ static int vAdjust(state_t *st) {
 	voronoi(nextOne, st);
     }
 
-    if (Verbose) {
-	fprintf(stderr, "Number of iterations = %u\n", iterCnt);
-	fprintf(stderr, "Number of increases = %u\n", increaseCnt);
-    }
+    GV_DEBUG("Number of iterations = %u", iterCnt);
+    GV_DEBUG("Number of increases = %u", increaseCnt);
 
     cleanup();
     return 1;
