@@ -24,7 +24,7 @@ enum {QUAD_TREE_HYBRID_SIZE = 10000};
 
 enum {QUAD_TREE_NONE = 0, QUAD_TREE_NORMAL, QUAD_TREE_FAST, QUAD_TREE_HYBRID};
 
-struct spring_electrical_control_struct {
+typedef struct {
   double p;/*a negativve real number default to -1. repulsive force = dist^p */
   double K;/* the natural distance. If K < 0, K will be set to the average distance of an edge */
   int multilevels;/* if <=1, single level */
@@ -48,26 +48,28 @@ struct spring_electrical_control_struct {
 			       0 (no action, default), 1 (penalty based method to make that kind of node close to the center of its neighbor), 
 			       1 (penalty based method to make that kind of node close to the old center of its neighbor),
 			       3 (two step process of overlap removal and straightening) */
-};
-
-typedef struct  spring_electrical_control_struct  *spring_electrical_control; 
+} spring_electrical_control;
 
 spring_electrical_control spring_electrical_control_new(void);
 void spring_electrical_control_print(spring_electrical_control ctrl);
 
-void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_control ctrl, double *x, int *flag);
-void spring_electrical_embedding_fast(int dim, SparseMatrix A0, spring_electrical_control ctrl, double *x, int *flag);
+void spring_electrical_embedding(int dim, SparseMatrix A0,
+                                 spring_electrical_control *ctrl, double *x,
+                                 int *flag);
+void spring_electrical_embedding_fast(int dim, SparseMatrix A0,
+                                      spring_electrical_control *ctrl,
+                                      double *x, int *flag);
 
 void multilevel_spring_electrical_embedding(int dim, SparseMatrix A0,
-                                            spring_electrical_control ctrl,
+                                            spring_electrical_control *ctrl,
                                             double *label_sizes, double *x,
                                             int n_edge_label_nodes,
                                             int *edge_label_nodes, int *flag);
 
-void spring_electrical_control_delete(spring_electrical_control ctrl);
-
 double average_edge_length(SparseMatrix A, int dim, double *coord);
 
-void spring_electrical_spring_embedding(int dim, SparseMatrix A, SparseMatrix D, spring_electrical_control ctrl, double *x, int *flag);
+void spring_electrical_spring_embedding(int dim, SparseMatrix A, SparseMatrix D,
+                                        spring_electrical_control *ctrl,
+                                        double *x, int *flag);
 
 void pcp_rotate(int n, int dim, double *x);
